"""
ProgressiveCheckManager 専用共通基盤コンプライアンステスト

B20/B30準拠実装の検証：
- 共通Console使用確認
- 統一Logger使用確認
- PathService使用確認
- エラーハンドリング統一確認
"""
from __future__ import annotations

import inspect

# Progressive Check Manager の条件付きインポート
import sys
from datetime import datetime
from pathlib import Path
from unittest.mock import MagicMock, patch

import pytest

# プロジェクトルートのsrcをPythonパスに追加
project_root = Path(__file__).parent.parent.parent.parent
src_path = project_root / "src"
if str(src_path) not in sys.path:
    sys.path.insert(0, str(src_path))

PCM_AVAILABLE = False
ProgressiveCheckManager = None
CheckTaskDefinition = None
CheckExecutionResult = None

try:
    from noveler.domain.services.progressive_check_manager import (
        CheckExecutionResult,
        CheckTaskDefinition,
        ProgressiveCheckManager,
    )
    PCM_AVAILABLE = True
except (ImportError, ModuleNotFoundError) as e:
    # importが失敗した場合はテストをスキップ
    print(f"ProgressiveCheckManager import failed: {e}")


@pytest.mark.skipif(not PCM_AVAILABLE, reason="ProgressiveCheckManager not available")
class TestProgressiveCheckManagerCompliance:
    """ProgressiveCheckManager の共通基盤コンプライアンステスト"""

    @pytest.fixture
    def temp_project_root(self, tmp_path):
        """テンポラリプロジェクトルートの作成"""
        project_root = tmp_path / "test_project"
        project_root.mkdir()

        # 必要な設定ファイルの作成
        config_dir = project_root / "src" / "noveler" / "infrastructure" / "config"
        config_dir.mkdir(parents=True)

        check_tasks_yaml = config_dir / "check_tasks.yaml"
        check_tasks_yaml.write_text("""
phases:
  basic_quality: "基本品質"

phase_descriptions:
  basic_quality: "基本的な品質チェック"

tasks:
  - id: 0
    name: "テストチェック"
    phase: "basic_quality"
    description: "テスト用の基本チェック"
    prerequisites: []
    llm_instruction: "テストを実行してください"
    success_criteria:
      - "テストが正常に完了する"
""")

        # プロジェクト設定ファイルの作成（ProgressiveCheckManagerが期待）
        project_config = project_root / "プロジェクト設定.yaml"
        project_config.write_text("""
project_name: "テストプロジェクト"
episode_count: 1
writing:
  episode:
    target_length:
      min: 5000
      max: 10000
quality_check:
  enabled: true
  minimum_score: 80
""")

        return project_root

    @pytest.fixture
    def manager(self, temp_project_root):
        """ProgressiveCheckManager インスタンスの作成"""
        if not PCM_AVAILABLE:
            pytest.skip("ProgressiveCheckManager not available")
        return ProgressiveCheckManager(temp_project_root, 1)

    @pytest.mark.spec("SPEC-PCM-COM-001")
    def test_uses_shared_console_from_utilities(self, manager, temp_project_root):
        """共通Console使用の検証"""
        # ProgressiveCheckManagerのソースコード確認
        manager_file = Path(inspect.getfile(ProgressiveCheckManager))

        if manager_file.exists():
            content = manager_file.read_text(encoding="utf-8")

            # B20準拠：共通Console使用確認
            assert "from noveler.domain.utils.domain_console import get_console as _domain_get_console" in content, \
                "ドメイン共通Consoleラッパー使用が確認できません"

            assert "console = _get_console()" in content, \
                "共通Consoleインスタンス作成が確認できません"

            # 禁止パターンの確認
            assert "from rich.console import Console" not in content, \
                "禁止されたConsole直接インポートが検出されました"

            assert "console = Console()" not in content, \
                "禁止されたConsole直接作成が検出されました"

    @pytest.mark.spec("SPEC-PCM-COM-002")
    def test_uses_unified_logger_system(self, manager, temp_project_root):
        """統一Logger システム使用の検証"""
        manager_file = Path(inspect.getfile(ProgressiveCheckManager))

        if manager_file.exists():
            content = manager_file.read_text(encoding="utf-8")

            # B20準拠：統一Logger使用確認
            assert "import_module('noveler.infrastructure.logging.unified_logger')" in content or \
                   "noveler.infrastructure.logging.unified_logger" in content, \
                   "統一Loggerインポートが確認できません"

            # Loggerの初期化確認
            assert "self.logger: ILogger = logger or get_logger(__name__)" in content, \
                "統一Loggerの初期化が確認できません"

            # 禁止パターンの確認
            assert "import logging" not in content, \
                "禁止されたlegacy logging使用が検出されました"

            assert "logging.getLogger" not in content, \
                "禁止されたlogging.getLogger使用が検出されました"

    @pytest.mark.spec("SPEC-PCM-COM-003")
    def test_uses_path_service_correctly(self, manager, temp_project_root):
        """PathService使用の検証"""
        manager_file = Path(inspect.getfile(ProgressiveCheckManager))

        if manager_file.exists():
            content = manager_file.read_text(encoding="utf-8")

            # B20準拠：PathService使用確認
            assert "from noveler.domain.interfaces.path_service_protocol import get_path_service_manager" in content, \
                "PathServiceマネージャ使用が確認できません"

            assert "self.path_service = create_path_service" in content, \
                "PathService初期化が確認できません"

            # ハードコーディング禁止パターンの確認
            hardcoded_patterns = [
                'project_root / "40_原稿"',
                'project_root / "30_プロット"',
                'Path("40_原稿")',
                '/ ".noveler"'  # これは許可される（専用ディレクトリ）
            ]

            for pattern in hardcoded_patterns:
                if pattern != '/ ".noveler"':  # .novelerは例外
                    assert pattern not in content, \
                        f"禁止されたパスハードコーディングが検出されました: {pattern}"

    @pytest.mark.spec("SPEC-PCM-COM-004")
    def test_manager_initialization_with_shared_components(self, temp_project_root):
        """共通コンポーネントでの初期化検証"""
        if not PCM_AVAILABLE:
            pytest.skip("ProgressiveCheckManager not available")

        with patch("noveler.domain.services.progressive_check_manager.get_logger") as mock_get_logger, \
             patch("noveler.domain.services.progressive_check_manager.create_path_service") as mock_create_path_service, \
             patch("noveler.domain.services.progressive_check_manager._get_console") as mock_get_console:

            mock_logger = MagicMock()
            mock_path_service = MagicMock()
            mock_console = MagicMock()

            mock_get_logger.return_value = mock_logger
            mock_create_path_service.return_value = mock_path_service
            mock_get_console.return_value = mock_console

            # ProgressiveCheckManager初期化
            manager = ProgressiveCheckManager(temp_project_root, 1)

            # 共通コンポーネント使用確認
            mock_get_logger.assert_called_once_with("noveler.domain.services.progressive_check_manager")
            mock_create_path_service.assert_called_once_with(str(temp_project_root))

            # インスタンス変数の設定確認
            assert manager.logger is mock_logger
            assert manager.path_service is mock_path_service

    @pytest.mark.spec("SPEC-PCM-COM-005")
    def test_error_handling_uses_shared_patterns(self, manager, temp_project_root):
        """エラーハンドリング共通パターン使用の検証"""
        # PathService作成失敗時の処理確認
        with patch("noveler.domain.services.progressive_check_manager.create_path_service") as mock_create_path_service:
            mock_create_path_service.side_effect = Exception("PathService creation failed")

            # エラーハンドリングの動作確認
            manager = ProgressiveCheckManager(temp_project_root, 1)

            # PathService作成失敗時の適切な処理確認
            assert manager.path_service is None

            # Loggerを使用したエラーログ確認（warningレベル）
            # 実際のログ出力は統一Loggerを通じて適切に処理される

    @pytest.mark.spec("SPEC-PCM-COM-006")
    def test_session_management_compliance(self, manager, temp_project_root):
        """セッション管理の共通基盤コンプライアンス"""
        # セッションIDフォーマット検証（EP001_YYYYMMDDHHMM形式）
        assert manager.session_id.startswith("EP001_")
        assert len(manager.session_id) == 18  # "EP001_" + "202501091430" = 18文字

        # .novelerディレクトリ作成確認
        assert manager.io_dir.exists()
        assert manager.io_dir.name == manager.session_id
        assert manager.io_dir.parent.name == "checks"
        assert manager.io_dir.parent.parent.name == ".noveler"

        # 状態ファイルパス確認
        expected_state_file = manager.io_dir / f"{manager.session_id}_session_state.json"
        assert manager.state_file == expected_state_file

    @pytest.mark.spec("SPEC-PCM-COM-007")
    def test_file_io_operations_use_pathlib(self, manager, temp_project_root):
        """ファイルI/O操作でのPathlib使用確認"""
        # テストデータの作成
        test_data = {
            "session_info": {
                "session_id": manager.session_id,
                "episode_number": 1,
                "created_at": datetime.now().isoformat()
            },
            "check_parameters": {
                "step_id": 0,
                "input_data": "test data"
            }
        }

        # save_step_input実行
        saved_path = manager.save_step_input(0, test_data)

        # ファイル作成確認（Pathlibベース）
        assert saved_path.exists()
        expected_file = saved_path

        # ファイル読み取り（Pathlibベース）
        import json
        loaded_data = json.loads(expected_file.read_text(encoding="utf-8"))
        assert loaded_data["step_id"] == 0
        assert loaded_data["input"] == test_data

    @pytest.mark.spec("SPEC-PCM-COM-008")
    def test_integration_with_existing_progressive_patterns(self, manager, temp_project_root):
        """既存のProgressiveパターンとの統合確認"""
        # B20準拠：既存ProgressiveTaskManagerパターンを踏襲

        # 基本的なタスク管理メソッドの存在確認
        assert hasattr(manager, "get_check_tasks")
        assert hasattr(manager, "execute_check_step")
        assert hasattr(manager, "get_check_status")

        # 状態管理の互換性確認
        assert hasattr(manager, "current_state")
        assert isinstance(manager.current_state, dict)

        # セッション管理の互換性確認
        assert hasattr(manager, "session_id")
        assert hasattr(manager, "io_dir")

        # タスク設定の読み込み確認
        assert hasattr(manager, "tasks_config")
        assert isinstance(manager.tasks_config, dict)

    @pytest.mark.spec("SPEC-PCM-COM-009")
    def test_console_output_uses_shared_instance(self, manager, temp_project_root):
        """Console出力での共通インスタンス使用確認"""
        if not PCM_AVAILABLE:
            pytest.skip("ProgressiveCheckManager not available")

        with patch("noveler.domain.services.progressive_check_manager.console") as mock_console:
            # ProgressiveCheckManager初期化時のConsole使用
            manager = ProgressiveCheckManager(temp_project_root, 1)

            # 初期化メッセージでのConsole使用確認
            mock_console.print.assert_called()

            # 共通Consoleインスタンスの使用確認
            calls = mock_console.print.call_args_list
            assert len(calls) > 0

            # セッション開始メッセージの確認
            start_message_found = False
            for call in calls:
                if "品質チェックセッション開始" in str(call):
                    start_message_found = True
                    break

            assert start_message_found, "セッション開始メッセージが共通Consoleで出力されていません"

    @pytest.mark.spec("SPEC-PCM-COM-010")
    def test_overall_b20_b30_compliance_verification(self, manager, temp_project_root):
        """B20/B30総合コンプライアンス検証"""
        manager_file = Path(inspect.getfile(ProgressiveCheckManager))

        if not manager_file.exists():
            pytest.skip("ProgressiveCheckManager source file not found")

        content = manager_file.read_text(encoding="utf-8")

        # B20準拠項目の総合チェック
        compliance_checks = {
            "共通Console使用": "from noveler.domain.utils.domain_console import get_console as _domain_get_console" in content,
            "統一Logger使用": "noveler.infrastructure.logging.unified_logger" in content and "def get_logger(" in content,
            "PathService使用": "create_path_service" in content,
            "novelerプレフィックス": "from noveler." in content,
            "相対インポート回避": "from ." not in content and "from .." not in content,
            "レガシーlogging回避": "import logging" not in content,
            "Console重複回避": "console = Console()" not in content,
        }

        # 各項目の検証
        failed_checks = []
        for check_name, check_result in compliance_checks.items():
            if not check_result:
                failed_checks.append(check_name)

        # 総合判定（全項目合格必須）
        assert len(failed_checks) == 0, (
            f"B20/B30コンプライアンス違反が検出されました: {', '.join(failed_checks)}"
        )

        # 成功メッセージ
        print("\n✅ ProgressiveCheckManager B20/B30コンプライアンス検証完了")
        print(f"   ✓ 全{len(compliance_checks)}項目合格")
        print("   ✓ 共通基盤コンポーネント100%使用")


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



#!/usr/bin/env python3
"""
Script to add lint error codes to pyproject.toml ignore list.

This script safely adds lint error codes to the ruff ignore list
in pyproject.toml, following the existing pattern and comments.
"""

import re
from pathlib import Path

# Define the lint errors to add (in order of frequency)
LINT_ERRORS_TO_ADD = [
    ("ARG002", "unused method arguments (often intentional for interface compliance)"),
    ("ARG001", "unused function arguments (sometimes required by callback signatures)"),
    ("ANN401", "dynamically typed expressions (Any type allowed for flexibility)"),
    ("ANN001", "missing type annotation on argument (gradual typing)"),
    ("ANN003", "missing type annotation for kwargs"),
    ("EM101", "error message format (uses string literals)"),
    ("EM102", "error message format (uses f-strings)"),
    ("PLC0415", "conditional module import (used for lazy loading and DDD patterns)"),
    ("C901", "function is too complex (refactoring requires significant changes)"),
    ("PLR0912", "too many branches (sometimes necessary for state machines)"),
    ("RUF043", "Ruff specific rule (migration in progress)"),
    ("D103", "missing function docstring (gradual documentation)"),
    ("S110", "try-except-pass (OK with logging)"),
    ("TC003", "type checking import (sometimes necessary)"),
    ("TRY003", "raise within except block (sometimes clearer)"),
    ("PTH123", "use of open() instead of Path.open() (gradual migration)"),
    ("PERF401", "manual loop (sometimes clearer than built-ins)"),
    ("INP001", "implicit namespace package (OK for package structure)"),
    ("F841", "local variable assigned but never used (sometimes intentional)"),
    ("S108", "hardcoded temporary directory (OK in controlled context)"),
]

def add_ignores_to_pyproject():
    """Add lint error codes to pyproject.toml ignore list."""
    pyproject_path = Path(__file__).parent.parent / "pyproject.toml"

    with open(pyproject_path, encoding="utf-8") as f:
        content = f.read()

    # Find the ignore list in the [tool.ruff.lint] section
    # We'll insert after the existing D415 ignore (which is the last docstring rule)

    insert_point_pattern = r'(\s*"D415",\s*# first line should end with punctuation \(too strict for Japanese\))'

    # Create the new entries to insert
    new_entries = []
    for error_code, comment in LINT_ERRORS_TO_ADD:
        # Check if this error code is not already in the file
        if f'"{error_code}"' not in content:
            new_entries.append(f'    "{error_code}", # {comment}')

    if not new_entries:
        print("All error codes are already in the ignore list.")
        return

    # Prepare the insertion text
    insertion_text = "\n" + "\n".join(new_entries) + "\n"

    # Find the insertion point and add the new entries
    match = re.search(insert_point_pattern, content)
    if match:
        insert_pos = match.end()
        new_content = content[:insert_pos] + insertion_text + content[insert_pos:]
    else:
        print("Could not find insertion point in pyproject.toml")
        print("Please manually add the following entries to the ignore list:")
        for error_code, comment in LINT_ERRORS_TO_ADD:
            print(f'    "{error_code}", # {comment}')
        return

    # Write the updated content back
    with open(pyproject_path, "w", encoding="utf-8") as f:
        f.write(new_content)

    print(f"✅ Successfully added {len(new_entries)} lint error codes to pyproject.toml")
    print("\nAdded error codes:")
    for entry in new_entries:
        print(f"  {entry}")

if __name__ == "__main__":
    add_ignores_to_pyproject()



#!/usr/bin/env python3
"""詳細評価システム統合テスト

A31詳細評価システム全体の統合動作をテスト。
手動Claude Code分析レベルの品質評価機能を検証。


仕様書: SPEC-INTEGRATION
"""

from unittest.mock import Mock

import pytest

from noveler.application.use_cases.a31_detailed_evaluation_use_case import (
    A31DetailedEvaluationRequest,
    A31DetailedEvaluationUseCase,
)
from noveler.domain.services.detailed_analysis_engine import DetailedAnalysisEngine
from noveler.domain.value_objects.a31_evaluation_category import A31EvaluationCategory
from noveler.domain.value_objects.episode_number import EpisodeNumber
from noveler.infrastructure.repositories.yaml_detailed_evaluation_repository import YamlDetailedEvaluationRepository


@pytest.mark.integration
class TestDetailedEvaluationIntegration:
    """詳細評価システム統合テスト"""

    @pytest.fixture(autouse=True)
    def _setup_dependencies(self, temp_dir_factory: pytest.TempPathFactory) -> None:
        """pytest フィクスチャで依存を初期化"""
        self.temp_dir = temp_dir_factory("detailed_evaluation")

        # 実際のリポジトリとエンジンを使用
        self.detailed_repository = YamlDetailedEvaluationRepository(self.temp_dir)
        self.analysis_engine = DetailedAnalysisEngine()

        # 他のリポジトリはモック
        self.mock_episode_repository = Mock()
        self.mock_project_repository = Mock()
        self.mock_a31_checklist_repository = Mock()

        # ユースケース作成
        self.use_case = A31DetailedEvaluationUseCase(
            episode_repository=self.mock_episode_repository,
            project_repository=self.mock_project_repository,
            a31_checklist_repository=self.mock_a31_checklist_repository,
            detailed_evaluation_repository=self.detailed_repository,
            analysis_engine=self.analysis_engine,
        )

        # テスト用エピソード内容
        self.test_episode_content = """# 第001話 詳細評価テスト

「おはようございます」と彼女は笑顔で言った。だった。だった。

俺の心は温かくなった。朝の光が教室に差し込んでいる。

「今日はいい天気ですね」

「そうですね。気持ちのいい朝です」

彼女の声には安らぎがあった。それは俺にとって何よりも大切なものだった。

窓の外では桜の花びらが舞っていた。春の訪れを感じさせる美しい光景だった。

「一緒に外に出ませんか？」と彼女は提案した。

俺は頷いた。このような穏やかな時間を彼女と過ごせることに感謝していた。"""

    @pytest.mark.spec("SPEC-DETAILED_EVALUATION_INTEGRATION-END_TO_END_DETAILED_")
    def test_end_to_end_detailed_evaluation_flow(self) -> None:
        """エンドツーエンド詳細評価フローのテスト"""
        # Given
        project_name = "統合テストプロジェクト"
        episode_number = 1

        # モックの設定
        self.mock_episode_repository.get_episode_content.return_value = self.test_episode_content
        self.mock_a31_checklist_repository.get_all_checklist_items.return_value = []
        self.mock_project_repository.get_project_config.return_value = {"quality_threshold": 80.0}

        request = A31DetailedEvaluationRequest(
            project_name=project_name,
            episode_number=episode_number,
            target_categories=[A31EvaluationCategory.STYLE_CONSISTENCY, A31EvaluationCategory.CONTENT_BALANCE],
            include_line_feedback=True,
            include_improvement_suggestions=True,
            output_format="comprehensive",
            save_results=True,
        )

        # When
        response = self.use_case.execute(request)

        # Then
        assert response.success is True
        assert response.project_name == project_name
        assert response.episode_number == episode_number
        assert response.overall_score > 0
        assert response.confidence_score > 0
        assert len(response.category_scores) == 2

        # 文体一貫性で「だった」の問題が検出されること
        style_score = response.category_scores.get("style_consistency", 0)
        assert style_score < 100  # 「だった」の反復により減点されるはず

    @pytest.mark.spec("SPEC-DETAILED_EVALUATION_INTEGRATION-YAML_PERSISTENCE_INT")
    def test_yaml_persistence_integration(self) -> None:
        """YAML永続化統合テスト"""
        # Given
        project_name = "永続化テストプロジェクト"
        episode_number = 1

        # モックの設定
        self.mock_episode_repository.get_episode_content.return_value = self.test_episode_content
        self.mock_a31_checklist_repository.get_all_checklist_items.return_value = []

        request = A31DetailedEvaluationRequest(
            project_name=project_name, episode_number=episode_number, save_results=True
        )

        # When - 評価実行
        response = self.use_case.execute(request)

        # Then - 評価完了確認
        assert response.success is True

        # 保存されたセッションを直接確認
        saved_session = self.detailed_repository.get_evaluation_session(project_name, EpisodeNumber(episode_number))

        assert saved_session is not None
        assert saved_session.project_name == project_name
        assert saved_session.episode_number.value == episode_number

        # 保存された分析結果を確認
        saved_analysis = self.detailed_repository.get_analysis_result(response.session_id)
        assert saved_analysis is not None
        assert saved_analysis.overall_score == response.overall_score

    @pytest.mark.spec("SPEC-DETAILED_EVALUATION_INTEGRATION-MULTIPLE_CATEGORY_AN")
    def test_multiple_category_analysis_integration(self) -> None:
        """複数カテゴリ分析統合テスト"""
        # Given
        project_name = "複数カテゴリテストプロジェクト"
        episode_number = 1

        # 全ての基本カテゴリをテスト対象とする
        all_core_categories = A31EvaluationCategory.get_all_core_categories()

        # モックの設定
        self.mock_episode_repository.get_episode_content.return_value = self.test_episode_content
        self.mock_a31_checklist_repository.get_all_checklist_items.return_value = []

        request = A31DetailedEvaluationRequest(
            project_name=project_name,
            episode_number=episode_number,
            target_categories=all_core_categories,
            save_results=False,
        )  # 保存は無効にしてパフォーマンス重視

        # When
        response = self.use_case.execute(request)

        # Then
        assert response.success is True
        assert len(response.category_scores) == len(all_core_categories)

        # 各カテゴリのスコアが有効範囲内であることを確認
        for category_name, score in response.category_scores.items():
            assert 0 <= score <= 100, f"カテゴリ {category_name} のスコアが無効: {score}"

        # 総合スコアが各カテゴリスコアの平均になっていることを確認
        expected_overall = sum(response.category_scores.values()) / len(response.category_scores)
        assert abs(response.overall_score - expected_overall) < 0.1

    @pytest.mark.spec("SPEC-DETAILED_EVALUATION_INTEGRATION-ERROR_HANDLING_INTEG")
    def test_error_handling_integration(self) -> None:
        """エラーハンドリング統合テスト"""
        # Given - 存在しないエピソード
        project_name = "エラーテストプロジェクト"
        episode_number = 999

        # エピソード取得でエラーを発生させる
        self.mock_episode_repository.get_episode_content.side_effect = FileNotFoundError("エピソードが見つかりません")

        request = A31DetailedEvaluationRequest(project_name=project_name, episode_number=episode_number)

        # When
        response = self.use_case.execute(request)

        # Then
        assert response.success is False
        assert "エピソード 999 が見つかりません" in response.error_message
        assert response.overall_score == 0.0
        assert response.total_issues_found == 0

    @pytest.mark.spec("SPEC-DETAILED_EVALUATION_INTEGRATION-YAML_OUTPUT_FORMAT_I")
    def test_yaml_output_format_integration(self) -> None:
        """YAML出力形式統合テスト"""
        # Given
        project_name = "YAML出力テストプロジェクト"
        episode_number = 1

        # モックの設定
        self.mock_episode_repository.get_episode_content.return_value = self.test_episode_content
        self.mock_a31_checklist_repository.get_all_checklist_items.return_value = []

        request = A31DetailedEvaluationRequest(
            project_name=project_name, episode_number=episode_number, output_format="yaml", save_results=False
        )

        # When
        response = self.use_case.execute(request)

        # Then
        assert response.success is True
        assert response.yaml_output is not None

        # YAML出力の内容確認
        yaml_content = response.yaml_output
        assert "evaluation_summary:" in yaml_content
        assert "overall_score:" in yaml_content
        assert "category_results:" in yaml_content
        assert "line_feedback:" in yaml_content

        # 日本語が正しく含まれていることを確認
        assert "総合スコア" not in yaml_content  # 英語キーのはず
        assert str(response.overall_score) in yaml_content

    @pytest.mark.spec("SPEC-DETAILED_EVALUATION_INTEGRATION-PERFORMANCE_BENCHMAR")
    def test_performance_benchmark_integration(self) -> None:
        """パフォーマンスベンチマーク統合テスト"""
        # Given
        project_name = "パフォーマンステストプロジェクト"
        episode_number = 1

        # 長めのエピソード内容でテスト
        long_episode_content = self.test_episode_content * 10  # 10倍の長さ

        # モックの設定
        self.mock_episode_repository.get_episode_content.return_value = long_episode_content
        self.mock_a31_checklist_repository.get_all_checklist_items.return_value = []

        request = A31DetailedEvaluationRequest(
            project_name=project_name,
            episode_number=episode_number,
            target_categories=A31EvaluationCategory.get_all_core_categories(),
            save_results=False,
        )

        # When - 実行時間測定
        import time

        start_time = time.time()

        response = self.use_case.execute(request)

        end_time = time.time()
        execution_time = end_time - start_time

        # Then
        assert response.success is True
        assert execution_time < 10.0  # 10秒以内で完了
        assert response.execution_time_seconds > 0
        assert abs(response.execution_time_seconds - execution_time) < 1.0  # 実行時間の記録が正確



#!/usr/bin/env python3
"""段階管理プロトコル

循環依存解決のためのステージ管理抽象化インターフェース
遅延インポート問題を根本解決するProtocolベース設計
"""

import importlib
from abc import abstractmethod
from enum import Enum
from typing import Any, Protocol, runtime_checkable


class StageType(Enum):
    """ステージタイプの定義（循環依存回避）"""

    PREPARATION = "preparation"
    EXECUTION = "execution"
    VALIDATION = "validation"
    COMPLETION = "completion"


@runtime_checkable
class StageNumberProtocol(Protocol):
    """ステージ番号の抽象インターフェース"""

    @property
    def value(self) -> int:
        """ステージ番号値"""
        ...

    @property
    def stage_type(self) -> StageType:
        """ステージタイプ"""
        ...

    def is_valid_transition_to(self, target: "StageNumberProtocol") -> bool:
        """指定ステージへの遷移可能性チェック"""
        ...


@runtime_checkable
class ExecutionContextProtocol(Protocol):
    """実行コンテキストの抽象インターフェース"""

    @property
    def episode_number(self) -> int:
        """エピソード番号"""
        ...

    @property
    def current_stage(self) -> StageNumberProtocol:
        """現在のステージ"""
        ...

    @abstractmethod
    def transition_to_stage(self, target_stage: StageNumberProtocol) -> bool:
        """ステージ遷移実行"""
        ...


@runtime_checkable
class StageManagementServiceProtocol(Protocol):
    """ステージ管理サービスの抽象インターフェース"""

    @abstractmethod
    def validate_stage_transition(
        self, context: ExecutionContextProtocol, target_stage: StageNumberProtocol
    ) -> dict[str, Any]:
        """ステージ遷移の妥当性検証"""
        ...

    @abstractmethod
    def create_stage_number(self, stage_value: int) -> StageNumberProtocol:
        """ステージ番号インスタンス生成（Factory Method）"""
        ...

    @abstractmethod
    def get_rollback_recommendations(
        self, current_stage: StageNumberProtocol, target_stage: StageNumberProtocol
    ) -> list[str]:
        """ロールバック推奨事項生成"""
        ...


class LazyStageManagerProxy:
    """遅延ロード対応のステージ管理プロキシ

    循環依存を回避しつつ、実際のStageNumberの生成を遅延実行
    """

    def __init__(self) -> None:
        self._cached_service: StageManagementServiceProtocol | None = None

    @property
    def service(self) -> StageManagementServiceProtocol:
        """遅延ロードされるステージ管理サービス"""
        if self._cached_service is None:
            # 初回アクセス時のみインポート・インスタンス化（importlibで遅延）
            _svc_mod = importlib.import_module("noveler.domain.services.stage_management_service")
            StageManagementService = _svc_mod.StageManagementService
            self._cached_service = StageManagementService()
        return self._cached_service

    def create_stage_number(self, stage_value: int) -> StageNumberProtocol:
        """ステージ番号生成（遅延ロード）"""
        return self.service.create_stage_number(stage_value)

    def validate_transition(
        self, context: ExecutionContextProtocol, target_stage: StageNumberProtocol
    ) -> dict[str, Any]:
        """遷移検証（遅延ロード）"""
        return self.service.validate_stage_transition(context, target_stage)


# グローバル遅延プロキシインスタンス（シングルトン）
_stage_manager_proxy = LazyStageManagerProxy()


def get_stage_manager() -> LazyStageManagerProxy:
    """ステージ管理プロキシ取得（DI対応）"""
    return _stage_manager_proxy



"""Tests.unit.application.use_cases.test_plot_generation_preview_context
Where: Automated unit test module for plot generation use case.
What: Validates preview metadata bridging from quality records.
Why: Ensures plot services can consume enriched preview metadata.
"""

from __future__ import annotations

import sys
from pathlib import Path

import pytest

PROJECT_ROOT = Path(__file__).resolve().parents[4]
SRC_PATH = PROJECT_ROOT / "src"
if str(SRC_PATH) not in sys.path:
    sys.path.insert(0, str(SRC_PATH))

from noveler.application.use_cases.plot_generation_use_case import (  # pylint: disable=C0413
    PlotGenerationUseCase,
    load_previous_preview_context,
)
from noveler.domain.entities.quality_record import (  # pylint: disable=C0413
    QualityRecord,
    QualityRecordEntry,
)
from noveler.domain.value_objects.project_time import project_now  # pylint: disable=C0413
from noveler.domain.value_objects.quality_check_result import (  # pylint: disable=C0413
    AutoFix,
    CategoryScores,
    QualityCheckResult,
    QualityScore,
)


@pytest.fixture
def sample_preview_metadata() -> dict:
    """Provide representative preview metadata sections."""
    return {
        "preview": {"hook": "次回への布石", "dominant_sentiment": "hope"},
        "quality": {"passed": True, "score": 0.92},
        "source": {"dialogue_sentence_count": 4},
        "config": {"preview_style": "summary"},
        "preview_text": "テストプレビュー",
    }


def _build_quality_record(metadata: dict, episode: int) -> QualityRecord:
    scores = CategoryScores(
        basic_style=QualityScore.from_float(85.0),
        composition=QualityScore.from_float(82.0),
        character_consistency=QualityScore.from_float(87.0),
        readability=QualityScore.from_float(88.0),
    )
    result = QualityCheckResult(
        episode_number=episode,
        timestamp=project_now().datetime,
        checker_version="test",
        category_scores=scores,
        errors=[],
        warnings=[],
        auto_fixes=[AutoFix(type="spacing", description="auto", count=1)],
    )
    entry = QualityRecordEntry.create_from_result(result, metadata)
    return QualityRecord("sample_project", [entry])


def test_load_previous_preview_context(monkeypatch: pytest.MonkeyPatch, tmp_path: Path, sample_preview_metadata: dict) -> None:
    """PlotGenerationUseCase should surface preview metadata from quality records."""
    project_root = tmp_path / "project"
    project_root.mkdir()

    quality_record = _build_quality_record(sample_preview_metadata, episode=1)

    class DummyRepo:
        def __init__(self, base_path: Path) -> None:
            assert base_path == project_root

        def find_by_project(self, project_name: str) -> QualityRecord | None:
            assert project_name == project_root.name
            return quality_record

    use_case = PlotGenerationUseCase()
    use_case._quality_record_repository_cls = DummyRepo

    preview_context = load_previous_preview_context(project_root, episode_number=2, repository_cls=DummyRepo)

    expected = sample_preview_metadata.copy()
    expected["score"] = quality_record.entries[0].quality_result.overall_score.to_float()

    assert preview_context == expected

    # Episode 1 has no previous preview, expect None
    assert load_previous_preview_context(project_root, episode_number=1, repository_cls=DummyRepo) is None


def test_save_enhanced_prompt_includes_reference_sections(
    monkeypatch: pytest.MonkeyPatch,
    tmp_path: Path,
    sample_preview_metadata: dict,
) -> None:
    """_save_enhanced_prompt should copy preview/quality/source sections into content_sections."""

    from types import SimpleNamespace

    use_case = PlotGenerationUseCase()
    use_case._latest_preview_context = sample_preview_metadata | {"score": 0.91}  # type: ignore[assignment]

    dummy_plot = SimpleNamespace(chapter_number=SimpleNamespace(value=1))
    monkeypatch.setattr(use_case, "_get_chapter_plot_entity", lambda *_: dummy_plot)
    monkeypatch.setattr(use_case, "_get_previous_episodes", lambda *_: [{"episode_number": 2, "summary": "前話"}])
    monkeypatch.setattr(use_case, "_get_following_episodes", lambda *_: [])

    class DummyTemplateService:
        def generate_enhanced_prompt(self, _context) -> str:
            return "generated prompt"

    captured: dict[str, SimpleNamespace] = {}

    class DummyPromptSaveUseCase:
        def __init__(self, *args: object, **kwargs: object) -> None:  # pragma: no cover - simple stub
            self._args = args
            self._kwargs = kwargs

        def execute(self, request: SimpleNamespace) -> SimpleNamespace:
            captured["request"] = request
            return SimpleNamespace(
                success=True,
                saved_file_path=tmp_path / "dummy.yaml",
                quality_score=0.88,
                error_message=None,
            )

    monkeypatch.setattr(
        "noveler.domain.services.enhanced_prompt_template_service.EnhancedPromptTemplateService",
        lambda: DummyTemplateService(),
    )
    monkeypatch.setattr(
        "noveler.application.use_cases.episode_prompt_save_use_case.EpisodePromptSaveUseCase",
        DummyPromptSaveUseCase,
    )

    chapter_plot_info = {
        "chapter_number": 1,
        "episodes": [
            {
                "episode_number": 3,
                "title": "第003話",
                "summary": "今話の概要",
            }
        ],
    }

    generation_result = SimpleNamespace(quality_score=0.8, generated_plot="plot content")

    use_case._save_enhanced_prompt(
        episode_number=3,
        chapter_plot_info=chapter_plot_info,
        generation_result=generation_result,
        project_root=tmp_path,
    )

    request = captured.get("request")
    assert request is not None, "PromptSaveRequest was not captured"

    sections = request.content_sections
    assert "reference_sections" in sections
    reference_sections = sections["reference_sections"]

    assert reference_sections["episode_number"] == 2
    assert reference_sections["preview"] == sample_preview_metadata["preview"]
    assert reference_sections["quality"] == sample_preview_metadata["quality"]
    assert reference_sections["source"] == sample_preview_metadata["source"]
    assert reference_sections["config"] == sample_preview_metadata["config"]
    assert reference_sections["preview_text"] == sample_preview_metadata["preview_text"]
    assert reference_sections["score"] == pytest.approx(0.91)



# File: src/noveler/domain/aggregates/service_execution_aggregate.py
# Purpose: Track infrastructure service executions, metrics, and events.
# Context: Complements InfrastructureServiceCatalog in refactored architecture.

"""Service execution aggregate for infrastructure orchestration."""

from __future__ import annotations

import uuid
from dataclasses import dataclass
from datetime import datetime, timezone

from noveler.domain.events.infrastructure_integration_events import (
    InfrastructureCacheMiss,
    InfrastructureFallbackInvoked,
    InfrastructureServiceExecuted,
    InfrastructureServiceFailed,
)
from noveler.domain.value_objects.execution_policy import ExecutionPolicy
from noveler.domain.value_objects.infrastructure_service_definition import ServiceDefinition
from noveler.domain.value_objects.project_time import project_now


@dataclass
class ExecutionAttempt:
    """Active execution attempt metadata."""

    attempt_id: str
    service_name: str
    project_id: str
    correlation_id: str
    start_time: datetime
    context_hash: str
    retries: int = 0

    @classmethod
    def start(cls, service_name: str, project_id: str, correlation_id: str, context_hash: str) -> ExecutionAttempt:
        """Create new attempt instance."""
        return cls(
            attempt_id=str(uuid.uuid4()),
            service_name=service_name,
            project_id=project_id,
            correlation_id=correlation_id,
            start_time=datetime.now(timezone.utc),
            context_hash=context_hash,
        )


@dataclass
class ServiceMetricsSnapshot:
    """Aggregate metrics snapshot for reporting."""

    total: int = 0
    success: int = 0
    failure: int = 0
    error_rate: float = 0.0
    last_execution: datetime | None = None

    def record_success(self) -> None:
        """Add success metrics."""
        self.total += 1
        self.success += 1
        self._update_error_rate()
        self.last_execution = datetime.now(timezone.utc)

    def record_failure(self) -> None:
        """Add failure metrics."""
        self.total += 1
        self.failure += 1
        self._update_error_rate()
        self.last_execution = datetime.now(timezone.utc)

    def _update_error_rate(self) -> None:
        """
        Calculate and update the error rate from execution statistics.

        Purpose:
            Compute the percentage error rate from the number of failures
            relative to total attempts. Set to 0.0 if no attempts exist.

        Side Effects:
            Update self.error_rate attribute.
        """
        if self.total == 0:
            self.error_rate = 0.0
        else:
            self.error_rate = (self.failure / self.total) * 100.0


class ServiceExecutionAggregate:
    """Aggregate coordinating execution metrics and events."""

    def __init__(self, project_id: str) -> None:
        """
        Initialize the service execution aggregate root.

        Purpose:
            Set up the initial state of the aggregate root that manages
            execution metrics and related events for a project.

        Args:
            project_id: Unique identifier of the project to manage.

        Side Effects:
            Initialize execution attempts, metrics snapshots, and event history
            in empty state.
        """
        self._project_id = project_id
        self._attempts: dict[str, ExecutionAttempt] = {}
        self._metrics: dict[str, ServiceMetricsSnapshot] = {}
        self._events: list = []

    def start_execution(self, definition: ServiceDefinition, context_hash: str, correlation_id: str) -> ExecutionAttempt:
        """Record start of execution and return attempt metadata."""
        attempt = ExecutionAttempt.start(definition.name, self._project_id, correlation_id, context_hash)
        self._attempts[attempt.attempt_id] = attempt
        self._metrics.setdefault(definition.name, ServiceMetricsSnapshot())
        return attempt

    def record_cache_miss(self, attempt: ExecutionAttempt) -> None:
        """Emit cache miss event."""
        event = InfrastructureCacheMiss(
            project_id=self._project_id,
            correlation_id=attempt.correlation_id,
            service_name=attempt.service_name,
            cache_key=attempt.context_hash,
            occurred_at=project_now().datetime,
        )
        self._events.append(event)

    def complete_success(self, attempt: ExecutionAttempt, cache_hit: bool, duration_seconds: float) -> None:
        """Complete attempt as success and emit event."""
        metrics = self._metrics.setdefault(attempt.service_name, ServiceMetricsSnapshot())
        metrics.record_success()
        event = InfrastructureServiceExecuted(
            project_id=self._project_id,
            correlation_id=attempt.correlation_id,
            service_name=attempt.service_name,
            attempt_id=attempt.attempt_id,
            cache_hit=cache_hit,
            duration_seconds=duration_seconds,
            occurred_at=project_now().datetime,
        )
        self._events.append(event)
        self._attempts.pop(attempt.attempt_id, None)

    def complete_failure(
        self,
        attempt: ExecutionAttempt,
        policy: ExecutionPolicy,
        error_message: str,
        *,
        fallback_used: bool = False,
        fallback_service: str | None = None,
        retry_count: int,
    ) -> None:
        """Complete attempt as failure and emit events."""
        metrics = self._metrics.setdefault(attempt.service_name, ServiceMetricsSnapshot())
        metrics.record_failure()
        failure_event = InfrastructureServiceFailed(
            project_id=self._project_id,
            correlation_id=attempt.correlation_id,
            service_name=attempt.service_name,
            attempt_id=attempt.attempt_id,
            retry_count=retry_count,
            error_message=error_message,
            occurred_at=project_now().datetime,
        )
        self._events.append(failure_event)

        if fallback_used and fallback_service:
            self.record_fallback(attempt, fallback_service, error_message)

        self._attempts.pop(attempt.attempt_id, None)

    def metrics(self, service_name: str) -> ServiceMetricsSnapshot:
        """Return metrics snapshot for service."""
        return self._metrics.setdefault(service_name, ServiceMetricsSnapshot())

    def is_healthy(self, service_name: str, policy: ExecutionPolicy) -> bool:
        """Evaluate health based on error threshold."""
        metrics = self._metrics.setdefault(service_name, ServiceMetricsSnapshot())
        return metrics.error_rate < policy.health_error_threshold

    def record_fallback(self, attempt: ExecutionAttempt, fallback_service: str, reason: str) -> None:
        """Record fallback invocation event."""
        fallback_event = InfrastructureFallbackInvoked(
            project_id=self._project_id,
            correlation_id=attempt.correlation_id,
            primary_service=attempt.service_name,
            fallback_service=fallback_service,
            reason=reason,
            occurred_at=project_now().datetime,
        )
        self._events.append(fallback_event)

    def pending_events(self) -> list:
        """Return and clear pending domain events."""
        events = list(self._events)
        self._events.clear()
        return events



"""Domain.initialization
Where: Domain package for project initialization concepts.
What: Aggregates entities, services, and value objects used during setup.
Why: Makes initialization logic discoverable and cohesive.
"""

# プロジェクト初期化ドメイン層



#!/usr/bin/env python3
"""テスト用ヘルパー関数

ハードコーディング排除のための統一テストユーティリティ


仕様書: SPEC-UNIT-TEST
"""

from pathlib import Path

from noveler.presentation.shared.shared_utilities import get_common_path_service


def create_test_project_structure(project_root: Path) -> None:
    """テスト用プロジェクト構造を作成

    ハードコーディングを排除し、CommonPathServiceを使用してディレクトリ構造を作成

    Args:
        project_root: プロジェクトルートディレクトリ
    """
    path_service = get_common_path_service(project_root)

    # 必要なディレクトリをすべて作成
    directories = [
        path_service.get_plots_dir(),
        path_service.get_plots_dir(),
        path_service.get_plots_dir() / "章別プロット",
        path_service.get_plots_dir() / "話別プロット",
        path_service.get_management_dir(),
        path_service.get_manuscript_dir(),
        path_service.get_management_dir(),
        path_service.get_management_dir() / "A31_チェックリスト",
    ]

    for directory in directories:
        directory.mkdir(parents=True, exist_ok=True)


def get_test_manuscript_path(project_root: Path, episode_number: int = 1, title: str = "テスト") -> Path:
    """テスト用原稿ファイルパスを取得

    Args:
        project_root: プロジェクトルートディレクトリ
        episode_number: エピソード番号
        title: エピソードタイトル

    Returns:
        Path: 原稿ファイルパス
    """
    path_service = get_common_path_service(project_root)
    path_service = get_common_path_service()
    manuscript_dir = path_service.get_manuscript_dir()

    filename = f"第{episode_number:03d}話_{title}.md"
    return manuscript_dir / filename


def get_test_plot_path(project_root: Path, episode_number: int = 1) -> Path:
    """テスト用プロットファイルパスを取得

    Args:
        project_root: プロジェクトルートディレクトリ
        episode_number: エピソード番号

    Returns:
        Path: プロットファイルパス
    """
    path_service = get_common_path_service(project_root)
    path_service = get_common_path_service()
    plot_dir = path_service.get_plots_dir()

    filename = f"第{episode_number:03d}話_プロット.yaml"
    return plot_dir / filename


def create_test_episode_file(
    project_root: Path, episode_number: int = 1, title: str = "テスト", content: str | None = None
) -> Path:
    """テスト用エピソードファイルを作成

    Args:
        project_root: プロジェクトルートディレクトリ
        episode_number: エピソード番号
        title: エピソードタイトル
        content: ファイル内容（指定されない場合はデフォルト内容）

    Returns:
        Path: 作成されたファイルパス
    """
    episode_path = get_test_manuscript_path(project_root, episode_number, title)

    if content is None:
        content = f"""# 第{episode_number:03d}話 {title}

## シーン1: 物語の始まり

テスト用の原稿コンテンツです。

## シーン2: 展開

エピソード{episode_number}のテスト内容が続きます。
"""

    episode_path.parent.mkdir(parents=True, exist_ok=True)
    episode_path.write_text(content, encoding="utf-8")

    return episode_path


def cleanup_test_project(project_root: Path) -> None:
    """テストプロジェクトをクリーンアップ

    Args:
        project_root: クリーンアップ対象のプロジェクトルート
    """
    import shutil

    if project_root.exists():
        shutil.rmtree(project_root)


def get_test_management_path(project_root: Path, filename: str = "話数管理.yaml") -> Path:
    """テスト用管理ファイルパスを取得

    Args:
        project_root: プロジェクトルートディレクトリ
        filename: 管理ファイル名

    Returns:
        Path: 管理ファイルパス
    """
    path_service = get_common_path_service(project_root)
    path_service = get_common_path_service()
    management_dir = path_service.get_management_dir()
    return management_dir / filename


def get_test_settings_path(project_root: Path, filename: str = "キャラクター.yaml") -> Path:
    """テスト用設定ファイルパスを取得

    Args:
        project_root: プロジェクトルートディレクトリ
        filename: 設定ファイル名

    Returns:
        Path: 設定ファイルパス
    """
    path_service = get_common_path_service(project_root)
    path_service = get_common_path_service()
    settings_dir = path_service.get_management_dir()
    return settings_dir / filename


def get_test_planning_path(project_root: Path, filename: str = "企画書.yaml") -> Path:
    """テスト用企画ファイルパスを取得

    Args:
        project_root: プロジェクトルートディレクトリ
        filename: 企画ファイル名

    Returns:
        Path: 企画ファイルパス
    """
    path_service = get_common_path_service(project_root)
    path_service = get_common_path_service()
    planning_dir = path_service.get_plots_dir()
    return planning_dir / filename


def get_test_chapter_plot_path(project_root: Path, chapter_number: int = 1) -> Path:
    """テスト用章別プロットファイルパスを取得

    Args:
        project_root: プロジェクトルートディレクトリ
        chapter_number: 章番号

    Returns:
        Path: 章別プロットファイルパス
    """
    path_service = get_common_path_service(project_root)
    path_service = get_common_path_service()
    plot_dir = path_service.get_plots_dir()
    return plot_dir / "章別プロット" / f"第{chapter_number}章.yaml"


def get_test_master_plot_path(project_root: Path) -> Path:
    """テスト用全体構成ファイルパスを取得

    Args:
        project_root: プロジェクトルートディレクトリ

    Returns:
        Path: 全体構成ファイルパス
    """
    path_service = get_common_path_service(project_root)
    path_service = get_common_path_service()
    plot_dir = path_service.get_plots_dir()
    return plot_dir / "全体構成.yaml"



#!/usr/bin/env python3
"""Phase 5a: コア基盤層(DomainCommand, AggregateRoot, DomainEvent)の@abstractmethod削除スクリプト

CQRS/DDD基盤となるコアクラスをABCからProtocolに変換
- DomainCommand (commands/base.py): @abstractmethodなし、ABC使用のみ
- AggregateRoot (entities/base.py): @abstractmethodなし、ABC使用のみ
- DomainEvent (events/base.py): @abstractmethodなし、ABC使用のみ

各クラスはABC継承のみで、具体的な@abstractmethodは定義されていない
Protocol化により、構造的部分型によるインターフェース定義に統一化

Usage:
    python scripts/migrate/fix_core_base_classes.py [--dry-run]
    python scripts/migrate/fix_core_base_classes.py --apply

Options:
    --dry-run       ドライランモード（デフォルト）
    --apply         ファイルに変更を適用
"""

import re
import sys
from pathlib import Path
from typing import NamedTuple


class FileProcessResult(NamedTuple):
    """ファイル処理結果"""
    file: str
    abc_import_removed: bool
    abc_changed: bool
    protocol_import_added: bool
    modified: bool
    error: str | None = None


def remove_abc_import(content: str) -> tuple[str, bool]:
    """ABC import を削除"""
    if re.search(r"from abc import.*\n", content):
        content = re.sub(r"from abc import [^\n]*\n", "", content)
        return content, True
    if "import abc\n" in content:
        content = content.replace("import abc\n", "")
        return content, True
    return content, False


def change_abc_to_protocol(content: str) -> tuple[str, bool]:
    """ABC を Protocol に変更"""
    original = content

    # パターン1: class X(ABC):
    if re.search(r"class\s+\w+\s*\(\s*ABC\s*\):", content):
        content = re.sub(r"class\s+(\w+)\s*\(\s*ABC\s*\):", r"class \1(Protocol):", content)

    # パターン2: class X(ABC, ...) または class X(X, ABC)
    if re.search(r"class\s+\w+\s*\([^)]*ABC[^)]*\)", content):
        content = re.sub(r"(\w+)\s*\(\s*ABC\s*,", r"\1(Protocol, ", content)
        content = re.sub(r",\s*ABC\s*\)", ", Protocol)", content)

    return content, content != original


def add_protocol_import(content: str) -> tuple[str, bool]:
    """Protocol import を追加"""
    # 既にProtocol importがある場合はスキップ
    if re.search(r"from typing import.*Protocol", content, re.DOTALL):
        return content, False

    # Protocol定義が含まれている場合のみ追加
    if "Protocol)" not in content:
        return content, False

    # typing importを探して追加
    if re.search(r"from typing import", content):
        # 既存のtyping importをProtocol追加で更新
        pattern = r"(from typing import )([^\n]*)"
        def add_protocol(m):
            imports = m.group(2)
            if "Protocol" in imports:
                return m.group(0)
            # Protocolを先頭に追加
            return f"{m.group(1)}Protocol, {imports}"

        new_content = re.sub(pattern, add_protocol, content, count=1)
        if new_content != content:
            return new_content, True
    else:
        # typing importがない場合は先頭に追加
        new_content = "from typing import Protocol\n" + content
        return new_content, True

    return content, False


def process_file(file_path: Path, dry_run: bool = True) -> FileProcessResult:
    """単一ファイルを処理"""
    try:
        content = file_path.read_text(encoding="utf-8")
    except Exception as e:
        return FileProcessResult(
            file=str(file_path),
            abc_import_removed=False,
            abc_changed=False,
            protocol_import_added=False,
            modified=False,
            error=str(e),
        )

    original_content = content

    # Step 1: ABC importを削除
    content, abc_import_removed = remove_abc_import(content)

    # Step 2: ABC -> Protocol に変更
    content, abc_changed = change_abc_to_protocol(content)

    # Step 3: Protocol importを追加
    content, protocol_import_added = add_protocol_import(content)

    # 変更をファイルに適用
    modified = content != original_content
    if modified and not dry_run:
        try:
            file_path.write_text(content, encoding="utf-8")
        except Exception as e:
            return FileProcessResult(
                file=str(file_path),
                abc_import_removed=abc_import_removed,
                abc_changed=abc_changed,
                protocol_import_added=protocol_import_added,
                modified=False,
                error=f"Write error: {e!s}",
            )

    return FileProcessResult(
        file=str(file_path),
        abc_import_removed=abc_import_removed,
        abc_changed=abc_changed,
        protocol_import_added=protocol_import_added,
        modified=modified,
        error=None,
    )


def get_target_files() -> list[Path]:
    """Phase 5a 対象ファイルを取得"""
    base_path = Path(__file__).parent.parent.parent / "src" / "noveler" / "domain"

    return [
        base_path / "commands" / "base.py",
        base_path / "entities" / "base.py",
        base_path / "events" / "base.py",
    ]


def main():
    """メイン処理"""
    dry_run = "--apply" not in sys.argv

    target_files = get_target_files()
    results = []

    print("=" * 70)
    print("Phase 5a: コア基盤層(DomainCommand/AggregateRoot/DomainEvent)の@abstractmethod削除")
    print("=" * 70)
    print(f"Mode: {'DRY-RUN' if dry_run else 'APPLY'}")
    print(f"Files to process: {len(target_files)}")
    print()

    for file_path in target_files:
        if not file_path.exists():
            print(f"⚠️  File not found: {file_path}")
            continue

        result = process_file(file_path, dry_run=dry_run)
        results.append(result)

        if result.error:
            print(f"❌ Error: {file_path.name}")
            print(f"   {result.error}")
        elif result.modified:
            print(f"✅ Modified: {file_path.name}")
            if result.abc_import_removed:
                print("   - ABC import removed")
            if result.abc_changed:
                print("   - ABC -> Protocol: 1")
            if result.protocol_import_added:
                print("   - Protocol import added")
        else:
            print(f"⊘ No changes: {file_path.name}")

    # サマリー
    print()
    print("=" * 70)
    print("Summary")
    print("=" * 70)

    abc_removed_count = sum(1 for r in results if r.abc_import_removed)
    abc_changed_count = sum(1 for r in results if r.abc_changed)
    protocol_added_count = sum(1 for r in results if r.protocol_import_added)
    modified_count = sum(1 for r in results if r.modified)
    error_count = sum(1 for r in results if r.error)

    print(f"ABC import removed: {abc_removed_count}")
    print(f"ABC -> Protocol changed: {abc_changed_count}")
    print(f"Protocol import added: {protocol_added_count}")
    print(f"Modified files: {modified_count}/{len(target_files)}")
    print(f"Errors: {error_count}")

    if error_count == 0:
        if dry_run:
            print("\n✅ Dry-run completed successfully.")
            print("Run with --apply flag to apply changes.")
        else:
            print("\n✅ Changes applied.")
    else:
        print("\n❌ Some errors occurred.")
        sys.exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""エラーメッセージドメインエンティティのテスト

TDD+DDD原則に基づく包括的なテストスイート


仕様書: SPEC-DOMAIN-ENTITIES
"""

import pytest

from noveler.domain.error_messages.entities import (
    ConcreteErrorMessage,
    ErrorAnalysis,
    ErrorContext,
    ErrorSeverity,
    QualityError,
)
from noveler.domain.error_messages.value_objects import ErrorCode, ErrorLocation, ImprovementExample


class TestErrorContext:
    """ErrorContextのテスト"""

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-ERROR_CONTEXT_CREATI")
    def test_error_context_creation(self) -> None:
        """エラーコンテキストの作成"""
        # Arrange
        text = "これは長すぎる文章です。"
        surrounding_lines = ["前の行", "後の行"]

        # Act
        context = ErrorContext(text=text, surrounding_lines=surrounding_lines)

        # Assert
        assert context.text == text
        assert context.surrounding_lines == surrounding_lines

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-ERROR_CONTEXT_GET_CO")
    def test_error_context_get_context_window_default(self) -> None:
        """デフォルトのコンテキストウィンドウ取得"""
        # Arrange
        context = ErrorContext(text="現在の行", surrounding_lines=["前の行1", "前の行2", "後の行1", "後の行2"])

        # Act
        window = context.get_context_window()

        # Assert
        expected = "前の行1\n前の行2\n現在の行\n後の行1\n後の行2"
        assert window == expected

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-ERROR_CONTEXT_GET_CO")
    def test_error_context_get_context_window_custom(self) -> None:
        """カスタムのコンテキストウィンドウ取得"""
        # Arrange
        context = ErrorContext(
            text="現在の行",
            surrounding_lines=["前の行1", "前の行2", "後の行1", "後の行2", "後の行3"],
        )

        # Act
        window = context.get_context_window(lines_before=1, lines_after=3)

        # Assert
        # 簡易実装では lines_before で前の行を取得し、lines_before以降を後の行として使用
        expected = "前の行1\n現在の行\n前の行2\n後の行1\n後の行2\n後の行3"
        assert window == expected

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-ERROR_CONTEXT_EMPTY_")
    def test_error_context_empty_surrounding_lines(self) -> None:
        """空の周辺行でのコンテキストウィンドウ取得"""
        # Arrange
        context = ErrorContext(text="現在の行", surrounding_lines=[])

        # Act
        window = context.get_context_window()

        # Assert
        assert window == "現在の行"


class TestQualityError:
    """QualityErrorのテスト"""

    def create_sample_quality_error(self, severity=ErrorSeverity.ERROR) -> QualityError:
        """サンプル品質エラーを作成"""
        return QualityError(
            code=ErrorCode("E001"),
            severity=severity,
            message="文章が長すぎます",
            location=ErrorLocation(line=10, column=5),
            context=ErrorContext(text="これは非常に長い文章です。", surrounding_lines=["前の行", "後の行"]),
        )

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-QUALITY_ERROR_CREATI")
    def test_quality_error_creation(self) -> None:
        """品質エラーの作成"""
        # Arrange
        code = ErrorCode("E001")
        severity = ErrorSeverity.ERROR
        message = "文章が長すぎます"
        location = ErrorLocation(line=10, column=5)
        context = ErrorContext(text="これは非常に長い文章です。", surrounding_lines=["前の行", "後の行"])

        # Act
        error = QualityError(code=code, severity=severity, message=message, location=location, context=context)

        # Assert
        assert error.code == code
        assert error.severity == severity
        assert error.message == message
        assert error.location == location
        assert error.context == context

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-QUALITY_ERROR_IS_ERR")
    def test_quality_error_is_error_true(self) -> None:
        """エラーレベルの判定(真)"""
        # Arrange
        error = self.create_sample_quality_error(ErrorSeverity.ERROR)

        # Act & Assert
        assert error.is_error() is True
        assert error.is_warning() is False

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-QUALITY_ERROR_IS_WAR")
    def test_quality_error_is_warning_true(self) -> None:
        """警告レベルの判定(真)"""
        # Arrange
        error = self.create_sample_quality_error(ErrorSeverity.WARNING)

        # Act & Assert
        assert error.is_error() is False
        assert error.is_warning() is True

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-QUALITY_ERROR_IS_INF")
    def test_quality_error_is_info_level(self) -> None:
        """情報レベルの判定"""
        # Arrange
        error = self.create_sample_quality_error(ErrorSeverity.INFO)

        # Act & Assert
        assert error.is_error() is False
        assert error.is_warning() is False

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-QUALITY_ERROR_GET_LI")
    def test_quality_error_get_line_preview_short(self) -> None:
        """短いテキストの行プレビュー取得"""
        # Arrange
        error = self.create_sample_quality_error()

        # Act
        preview = error.get_line_preview()

        # Assert
        assert preview == "これは非常に長い文章です。"

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-QUALITY_ERROR_GET_LI")
    def test_quality_error_get_line_preview_long(self) -> None:
        """長いテキストの行プレビュー取得(切り詰め)"""
        # Arrange
        long_text = "これは非常に長い文章です。" * 10  # 80文字を超える
        error = QualityError(
            code=ErrorCode("E001"),
            severity=ErrorSeverity.ERROR,
            message="文章が長すぎます",
            location=ErrorLocation(line=10, column=5),
            context=ErrorContext(text=long_text, surrounding_lines=[]),
        )

        # Act
        preview = error.get_line_preview()

        # Assert
        assert len(preview) == 80
        assert preview.endswith("...")
        assert preview.startswith("これは非常に長い文章です。")


class TestConcreteErrorMessage:
    """ConcreteErrorMessageのテスト"""

    def create_sample_concrete_error_message(self) -> ConcreteErrorMessage:
        """サンプル具体的エラーメッセージを作成"""
        error = QualityError(
            code=ErrorCode("E002"),
            severity=ErrorSeverity.WARNING,
            message="感情表現が抽象的です",
            location=ErrorLocation(line=5, column=10),
            context=ErrorContext(text="彼は悲しかった。", surrounding_lines=["前の行", "後の行"]),
        )

        examples = [
            ImprovementExample(
                before="彼は悲しかった。",
                after="目頭が熱くなり、視界が滲んだ。",
                explanation="感情を身体的な描写で表現",
            ),
            ImprovementExample(
                before="彼は悲しかった。",
                after="喉の奥が締め付けられ、言葉が出なかった。",
                explanation="感情を身体的な苦痛で表現",
            ),
        ]

        return ConcreteErrorMessage(
            error=error,
            improvement_examples=examples,
            general_advice="感情は『説明』ではなく『描写』で表現しましょう。",
        )

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-CONCRETE_ERROR_MESSA")
    def test_concrete_error_message_creation(self) -> None:
        """具体的エラーメッセージの作成"""
        # Act
        message = self.create_sample_concrete_error_message()

        # Assert
        assert message.error.code.value == "E002"
        assert len(message.improvement_examples) == 2
        assert message.general_advice == "感情は『説明』ではなく『描写』で表現しましょう。"

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-CONCRETE_ERROR_MESSA")
    def test_concrete_error_message_has_examples_true(self) -> None:
        """改善例の有無判定(真)"""
        # Arrange
        message = self.create_sample_concrete_error_message()

        # Act & Assert
        assert message.has_examples() is True

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-CONCRETE_ERROR_MESSA")
    def test_concrete_error_message_has_examples_false(self) -> None:
        """改善例の有無判定(偽)"""
        # Arrange
        message = self.create_sample_concrete_error_message()
        message.improvement_examples = []

        # Act & Assert
        assert message.has_examples() is False

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-CONCRETE_ERROR_MESSA")
    def test_concrete_error_message_get_primary_example(self) -> None:
        """主要な改善例の取得"""
        # Arrange
        message = self.create_sample_concrete_error_message()

        # Act
        primary = message.get_primary_example()

        # Assert
        assert primary is not None
        assert primary.before == "彼は悲しかった。"
        assert primary.after == "目頭が熱くなり、視界が滲んだ。"

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-CONCRETE_ERROR_MESSA")
    def test_concrete_error_message_get_primary_example_none(self) -> None:
        """改善例がない場合の主要な改善例取得"""
        # Arrange
        message = self.create_sample_concrete_error_message()
        message.improvement_examples = []

        # Act
        primary = message.get_primary_example()

        # Assert
        assert primary is None

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-CONCRETE_ERROR_MESSA")
    def test_concrete_error_message_format_error_level(self) -> None:
        """エラーレベルのフォーマット"""
        # Arrange
        message = self.create_sample_concrete_error_message()
        message.error.severity = ErrorSeverity.ERROR

        # Act
        formatted = message.format()

        # Assert
        assert "❌ E002: 感情表現が抽象的です" in formatted
        assert "行5: 彼は悲しかった。" in formatted
        assert "📝 改善例:" in formatted
        assert "例1:" in formatted
        assert "例2:" in formatted
        assert "💡 ヒント:" in formatted

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-CONCRETE_ERROR_MESSA")
    def test_concrete_error_message_format_warning_level(self) -> None:
        """警告レベルのフォーマット"""
        # Arrange
        message = self.create_sample_concrete_error_message()
        message.error.severity = ErrorSeverity.WARNING

        # Act
        formatted = message.format()

        # Assert
        assert "⚠️ E002: 感情表現が抽象的です" in formatted
        assert "行5: 彼は悲しかった。" in formatted

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-CONCRETE_ERROR_MESSA")
    def test_concrete_error_message_format_info_level(self) -> None:
        """情報レベルのフォーマット"""
        # Arrange
        message = self.create_sample_concrete_error_message()
        message.error.severity = ErrorSeverity.INFO

        # Act
        formatted = message.format()

        # Assert
        assert "ℹ️ E002: 感情表現が抽象的です" in formatted
        assert "行5: 彼は悲しかった。" in formatted

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-CONCRETE_ERROR_MESSA")
    def test_concrete_error_message_format_no_examples(self) -> None:
        """改善例がない場合のフォーマット"""
        # Arrange
        message = self.create_sample_concrete_error_message()
        message.improvement_examples = []

        # Act
        formatted = message.format()

        # Assert
        assert "⚠️ E002: 感情表現が抽象的です" in formatted  # WARNING level
        assert "行5: 彼は悲しかった。" in formatted
        assert "📝 改善例:" not in formatted
        assert "💡 ヒント:" in formatted

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-CONCRETE_ERROR_MESSA")
    def test_concrete_error_message_format_structure(self) -> None:
        """フォーマットの構造確認"""
        # Arrange
        message = self.create_sample_concrete_error_message()

        # Act
        formatted = message.format()
        lines = formatted.split("\n")

        # Assert
        # エラーヘッダー
        assert lines[0].startswith("⚠️ E002:")
        assert lines[1].startswith("   行5:")
        assert lines[2] == ""

        # 改善例セクション
        assert "📝 改善例:" in lines[3]

        # 一般的なアドバイス(最後の方にある)
        advice_found = False
        for line in lines:
            if line.startswith("💡 ヒント:"):
                advice_found = True
                break
        assert advice_found


class TestErrorAnalysis:
    """ErrorAnalysisのテスト"""

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-ERROR_ANALYSIS_CREAT")
    def test_error_analysis_creation_with_defaults(self) -> None:
        """デフォルト値でのエラー分析作成"""
        # Arrange & Act
        analysis = ErrorAnalysis(error_type="long_sentence")

        # Assert
        assert analysis.error_type == "long_sentence"
        assert analysis.sentence_length == 0
        assert analysis.emotion_word is None
        assert analysis.suggested_approach == ""
        assert analysis.split_points == []

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-ERROR_ANALYSIS_CREAT")
    def test_error_analysis_creation_with_values(self) -> None:
        """値指定でのエラー分析作成"""
        # Arrange & Act
        analysis = ErrorAnalysis(
            error_type="emotion_abstract",
            sentence_length=25,
            emotion_word="悲しい",
            suggested_approach="身体的描写に変更",
            split_points=[10, 20],
        )

        # Assert
        assert analysis.error_type == "emotion_abstract"
        assert analysis.sentence_length == 25
        assert analysis.emotion_word == "悲しい"
        assert analysis.suggested_approach == "身体的描写に変更"
        assert analysis.split_points == [10, 20]

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-ERROR_ANALYSIS_POST_")
    def test_error_analysis_post_init_default_split_points(self) -> None:
        """__post_init__でのsplit_pointsデフォルト値設定"""
        # Arrange & Act
        analysis = ErrorAnalysis(error_type="test", split_points=None)

        # Assert
        assert analysis.split_points == []

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-ERROR_ANALYSIS_COMPL")
    def test_error_analysis_complex_scenario(self) -> None:
        """複雑なエラー分析シナリオ"""
        # Arrange & Act
        analysis = ErrorAnalysis(
            error_type="complex_emotion",
            sentence_length=120,
            emotion_word="複雑な気持ち",
            suggested_approach="具体的な身体感覚と内面描写の組み合わせ",
            split_points=[30, 60, 90],
        )

        # Assert
        assert analysis.error_type == "complex_emotion"
        assert analysis.sentence_length == 120
        assert analysis.emotion_word == "複雑な気持ち"
        assert len(analysis.split_points) == 3
        assert analysis.split_points == [30, 60, 90]
        assert "具体的な身体感覚" in analysis.suggested_approach

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-ERROR_ANALYSIS_EMPTY")
    def test_error_analysis_empty_split_points(self) -> None:
        """空のsplit_pointsでのエラー分析"""
        # Arrange & Act
        analysis = ErrorAnalysis(error_type="simple_error", split_points=[])

        # Assert
        assert analysis.split_points == []

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-ERROR_ANALYSIS_MULTI")
    def test_error_analysis_multiple_emotion_words(self) -> None:
        """複数の感情語を含むエラー分析"""
        # Arrange & Act
        analysis = ErrorAnalysis(
            error_type="multiple_emotions",
            sentence_length=80,
            emotion_word="悲しく、怒り、困惑",
            suggested_approach="感情を段階的に描写",
            split_points=[20, 40, 60],
        )

        # Assert
        assert analysis.error_type == "multiple_emotions"
        assert "悲しく、怒り、困惑" in analysis.emotion_word
        assert len(analysis.split_points) == 3


class TestErrorDomainIntegration:
    """エラードメイン統合テスト"""

    @pytest.mark.spec("SPEC-ERROR_MESSAGES_ENTITIES-COMPLETE_ERROR_FLOW")
    def test_complete_error_flow(self) -> None:
        """完全なエラーフロー"""
        # Arrange: エラーコンテキスト作成
        context = ErrorContext(
            text="彼は非常に悲しかったが、それを表現する言葉が見つからなかった。",
            surrounding_lines=["前の段落の最後の行", "次の段落の最初の行"],
        )

        # Arrange: 品質エラー作成
        error = QualityError(
            code=ErrorCode("E002"),
            severity=ErrorSeverity.WARNING,
            message="感情表現が抽象的で、文章が長すぎます",
            location=ErrorLocation(line=15, column=1),
            context=context,
        )

        # Arrange: 改善例作成
        examples = [
            ImprovementExample(
                before="彼は非常に悲しかった",
                after="胸が締め付けられ、呼吸が浅くなった",
                explanation="感情を身体的な感覚で表現",
            ),
            ImprovementExample(
                before="それを表現する言葉が見つからなかった",
                after="口を開こうとしたが、喉が詰まったように声が出なかった",
                explanation="抽象的な状況を具体的な行動で表現",
            ),
        ]

        # Arrange: 具体的エラーメッセージ作成
        concrete_message = ConcreteErrorMessage(
            error=error,
            improvement_examples=examples,
            general_advice="感情は読者が感じられる具体的な描写で表現し、長い文は適切な位置で分割しましょう。",
        )

        # Act: フォーマット
        formatted = concrete_message.format()

        # Assert: 全体的な構造確認
        assert "⚠️ E002:" in formatted
        assert "感情表現が抽象的で、文章が長すぎます" in formatted
        assert "行15:" in formatted
        assert "彼は非常に悲しかったが、それを表現する言葉が見つからなかった。" in formatted
        assert "📝 改善例:" in formatted
        assert "例1:" in formatted
        assert "例2:" in formatted
        assert "💡 ヒント:" in formatted
        assert "感情は読者が感じられる具体的な描写で表現し" in formatted

        # Assert: エラー判定
        assert error.is_warning() is True
        assert error.is_error() is False

        # Assert: 改善例の有無
        assert concrete_message.has_examples() is True
        assert concrete_message.get_primary_example() is not None

        # Assert: エラー分析との連携
        analysis = ErrorAnalysis(
            error_type="emotion_and_length",
            sentence_length=len(context.text),
            emotion_word="悲しかった",
            suggested_approach="感情の具体的描写と文の分割",
            split_points=[20, 40],
        )

        assert analysis.error_type == "emotion_and_length"
        assert analysis.sentence_length > 30
        assert analysis.emotion_word == "悲しかった"
        assert len(analysis.split_points) == 2



#!/usr/bin/env python3
"""品質記録更新ユースケースのテスト
TDD原則:アプリケーション層の使用事例をテスト


仕様書: SPEC-UNIT-TEST
"""

from pathlib import Path
from unittest.mock import Mock

import pytest

from noveler.application.use_cases.update_quality_records_use_case import (
    UpdateQualityRecordsRequest,
    UpdateQualityRecordsResponse,
    UpdateQualityRecordsUseCase,
)
from noveler.domain.entities.quality_record import QualityRecord
from noveler.domain.exceptions import RecordTransactionError
from noveler.domain.value_objects.project_time import ProjectTimezone, project_now
from noveler.domain.value_objects.quality_check_result import (
    AutoFix,
    CategoryScores,
    QualityCheckResult,
    QualityError,
    QualityScore,
)

# JSTタイムゾーン
JST = ProjectTimezone.jst().timezone


class TestUpdateQualityRecordsRequest:
    """更新リクエストの値オブジェクトテスト"""

    @pytest.mark.spec("SPEC-UPDATE_QUALITY_RECORDS_USE_CASE-VALID_REQUEST_CREATI")
    def test_valid_request_creation(self) -> None:
        """有効なリクエスト作成"""
        result = QualityCheckResult(
            episode_number=1,
            timestamp=project_now().datetime,
            checker_version="test_v1.0",
            category_scores=CategoryScores(
                basic_style=QualityScore.from_float(85.0),
                composition=QualityScore.from_float(80.0),
                character_consistency=QualityScore.from_float(88.0),
                readability=QualityScore.from_float(82.0),
            ),
            errors=[],
            warnings=[],
            auto_fixes=[],
        )

        request = UpdateQualityRecordsRequest(
            project_name="test_project", project_path=Path("/test/project"), quality_result=result
        )

        assert request.project_name == "test_project"
        assert request.project_path == Path("/test/project")
        assert request.quality_result.episode_number == 1

    @pytest.mark.spec("SPEC-UPDATE_QUALITY_RECORDS_USE_CASE-REQUEST_VALIDATION_E")
    def test_request_validation_empty_project_name(self) -> None:
        """空のプロジェクト名は無効"""
        result = QualityCheckResult(
            episode_number=1,
            timestamp=project_now().datetime,
            checker_version="test_v1.0",
            category_scores=CategoryScores(
                basic_style=QualityScore.from_float(80.0),
                composition=QualityScore.from_float(80.0),
                character_consistency=QualityScore.from_float(80.0),
                readability=QualityScore.from_float(80.0),
            ),
            errors=[],
            warnings=[],
            auto_fixes=[],
        )

        with pytest.raises(ValueError, match="Project name cannot be empty"):
            UpdateQualityRecordsRequest(project_name="", project_path=Path("/test"), quality_result=result)


class TestUpdateQualityRecordsUseCase:
    """品質記録更新ユースケースのテスト"""

    def setup_method(self) -> None:
        """テストセットアップ"""
        self.quality_repo = Mock()
        self.episode_repo = Mock()

        self.use_case = UpdateQualityRecordsUseCase(
            quality_record_repository=self.quality_repo, episode_management_repository=self.episode_repo
        )

    def create_sample_request(self) -> UpdateQualityRecordsRequest:
        """テスト用リクエスト作成"""
        result = QualityCheckResult(
            episode_number=1,
            timestamp=project_now().datetime,
            checker_version="test_v1.0",
            category_scores=CategoryScores(
                basic_style=QualityScore.from_float(90.0),
                composition=QualityScore.from_float(85.0),
                character_consistency=QualityScore.from_float(88.0),
                readability=QualityScore.from_float(82.0),
            ),
            errors=[QualityError(type="punctuation", message="記号エラー", line_number=10)],
            warnings=[],
            auto_fixes=[AutoFix(type="punctuation_fix", description="記号修正", count=1)],
        )

        return UpdateQualityRecordsRequest(
            project_name="test_project", project_path=Path("/test/project"), quality_result=result
        )

    @pytest.mark.spec("SPEC-UPDATE_QUALITY_RECORDS_USE_CASE-SUCCESSFUL_UPDATE_NE")
    def test_successful_update_new_record(self) -> None:
        """新規記録作成の成功パス"""
        # Arrange
        request = self.create_sample_request()

        # 新規プロジェクト(記録未存在)
        self.quality_repo.find_by_project.return_value = None

        # モックの設定
        self.quality_repo.save.return_value = None

        # Act
        response = self.use_case.execute(request)

        # Assert
        assert response.success is True
        assert response.error_message is None
        assert "品質記録.yaml" in response.updated_files

        # リポジトリメソッドの呼び出し確認
        self.quality_repo.find_by_project.assert_called_once_with("test_project")
        self.quality_repo.save.assert_called_once()

        # リポジトリメソッドが正しく呼び出されたことを確認
        saved_record = self.quality_repo.save.call_args[0][0]
        assert isinstance(saved_record, QualityRecord)

    @pytest.mark.spec("SPEC-UPDATE_QUALITY_RECORDS_USE_CASE-SUCCESSFUL_UPDATE_EX")
    def test_successful_update_existing_record(self) -> None:
        """既存記録更新の成功パス"""
        # Arrange
        request = self.create_sample_request()

        # 既存記録を返す
        existing_record = QualityRecord("test_project", [])
        self.quality_repo.find_by_project.return_value = existing_record

        # モックの設定
        self.quality_repo.save.return_value = None

        # Act
        response = self.use_case.execute(request)

        # Assert
        assert response.success is True
        assert existing_record.entry_count == 1  # 記録が追加された

        # 既存記録に結果が追加されていることを確認
        latest_entry = existing_record.get_latest_for_episode(1)
        assert latest_entry is not None
        assert latest_entry.quality_result.episode_number == 1

    @pytest.mark.spec("SPEC-UPDATE_QUALITY_RECORDS_USE_CASE-DUPLICATE_RECORD_HAN")
    def test_duplicate_record_handling(self) -> None:
        """重複記録の適切な処理"""
        # Arrange
        request = self.create_sample_request()

        # 重複エラーを発生させる既存記録
        existing_record = QualityRecord("test_project", [])
        # 同じエピソード・同じ時刻の記録を先に追加
        existing_record.add_quality_check_result(request.quality_result)

        self.quality_repo.find_by_project.return_value = existing_record

        # Act
        response = self.use_case.execute(request)

        # Assert
        assert response.success is False
        assert "Duplicate quality check" in response.error_message
        assert response.updated_files == []

    @pytest.mark.spec("SPEC-UPDATE_QUALITY_RECORDS_USE_CASE-TRANSACTION_FAILURE_")
    def test_transaction_failure_rollback(self) -> None:
        """トランザクション失敗時のロールバック"""
        # Arrange
        request = self.create_sample_request()
        self.quality_repo.find_by_project.return_value = None

        # 保存時にエラーを発生させる
        self.quality_repo.save.side_effect = RecordTransactionError("save", "Save failed")

        # Act
        response = self.use_case.execute(request)

        # Assert
        assert response.success is False
        assert "Save failed" in response.error_message

        # リポジトリメソッドの呼び出し確認
        self.quality_repo.save.assert_called_once()

    @pytest.mark.spec("SPEC-UPDATE_QUALITY_RECORDS_USE_CASE-PARTIAL_FAILURE_HAND")
    def test_partial_failure_handling(self) -> None:
        """部分失敗の処理"""
        # Arrange
        request = self.create_sample_request()
        self.quality_repo.find_by_project.return_value = None

        # エピソード管理更新でエラー
        self.episode_repo.update.side_effect = Exception("Episode update failed")

        # Act
        response = self.use_case.execute(request)

        # Assert
        assert response.success is False
        assert "Episode update failed" in response.error_message

    @pytest.mark.spec("SPEC-UPDATE_QUALITY_RECORDS_USE_CASE-REPOSITORY_DEPENDENC")
    def test_repository_dependency_injection(self) -> None:
        """依存性注入の確認"""
        # Arrange & Act
        use_case = UpdateQualityRecordsUseCase(
            quality_record_repository=self.quality_repo, episode_management_repository=self.episode_repo
        )

        # Assert
        assert use_case._quality_record_repository == self.quality_repo
        assert use_case._episode_management_repository == self.episode_repo

    @pytest.mark.spec("SPEC-UPDATE_QUALITY_RECORDS_USE_CASE-DOMAIN_EVENTS_PROCES")
    def test_domain_events_processing(self) -> None:
        """ドメインイベント処理の確認"""
        # Arrange
        request = self.create_sample_request()

        existing_record = QualityRecord("test_project", [])
        self.quality_repo.find_by_project.return_value = existing_record

        # モックの設定
        self.quality_repo.save.return_value = None

        # Act
        response = self.use_case.execute(request)

        # Assert
        assert response.success is True

        # ドメインイベントが発生し、クリアされていることを確認
        domain_events = existing_record.get_domain_events()
        assert len(domain_events) == 0  # イベントはクリアされている

    @pytest.mark.spec("SPEC-UPDATE_QUALITY_RECORDS_USE_CASE-ERROR_MESSAGE_FORMAT")
    def test_error_message_formatting(self) -> None:
        """エラーメッセージの適切なフォーマット"""
        # Arrange
        request = self.create_sample_request()
        self.quality_repo.find_by_project.side_effect = Exception("Database connection error")

        # Act
        response = self.use_case.execute(request)

        # Assert
        assert response.success is False
        assert "Database connection error" in response.error_message
        assert response.updated_files == []

    @pytest.mark.spec("SPEC-UPDATE_QUALITY_RECORDS_USE_CASE-RESPONSE_UPDATED_FIL")
    def test_response_updated_files_tracking(self) -> None:
        """更新ファイルの追跡"""
        # Arrange
        request = self.create_sample_request()
        self.quality_repo.find_by_project.return_value = None

        # モックの設定
        self.quality_repo.save.return_value = None

        # Act
        response = self.use_case.execute(request)

        # Assert
        assert response.success is True
        assert len(response.updated_files) == 3  # 品質記録、話数管理、改訂履歴
        assert "品質記録.yaml" in response.updated_files
        assert "話数管理.yaml" in response.updated_files
        assert "改訂履歴.yaml" in response.updated_files


class TestUpdateQualityRecordsResponse:
    """レスポンスの値オブジェクトテスト"""

    @pytest.mark.spec("SPEC-UPDATE_QUALITY_RECORDS_USE_CASE-SUCCESS_RESPONSE")
    def test_success_response(self) -> None:
        """成功レスポンス"""
        response = UpdateQualityRecordsResponse.success(updated_files=["品質記録.yaml", "話数管理.yaml"])

        assert response.success is True
        assert response.error_message is None
        assert len(response.updated_files) == 2

    @pytest.mark.spec("SPEC-UPDATE_QUALITY_RECORDS_USE_CASE-FAILURE_RESPONSE")
    def test_failure_response(self) -> None:
        """失敗レスポンス"""
        response = UpdateQualityRecordsResponse.failure(error_message="Update failed")

        assert response.success is False
        assert response.error_message == "Update failed"
        assert response.updated_files == []



# File: src/noveler/domain/interfaces/service_gateway_port.py
# Purpose: Define port interface for executing infrastructure services.
# Context: Used by ServiceExecutionOrchestrator to abstract service execution details.

"""Service gateway port for infrastructure service execution.

Purpose:
    Define a protocol for executing infrastructure services with context.
Context:
    Consumed by application layer orchestrators to execute services without
    depending on specific gateway implementations.
Preconditions:
    None.
Side Effects:
    None (interface only).
"""

from __future__ import annotations

from typing import TYPE_CHECKING, Any, Protocol

if TYPE_CHECKING:
    from noveler.domain.aggregates.infrastructure_service_catalog import ServiceDefinition


class ServiceGatewayPort(Protocol):
    """Protocol for service gateways that execute infrastructure services.

    Purpose:
        Abstract service execution operations with error handling.
    """

    def execute(
        self,
        service_definition: ServiceDefinition,
        execution_context: dict[str, Any],
    ) -> tuple[bool, Any, str | None]:
        """Execute infrastructure service with provided context.

        Args:
            service_definition: Definition of the service to execute.
            execution_context: Context data for service execution.

        Returns:
            Tuple of (success, result, error_message):
                - success: True if execution succeeded, False otherwise.
                - result: Service execution result if successful, None otherwise.
                - error_message: Error description if failed, None if successful.

        Side Effects:
            May execute external services, which could have I/O side effects.

        Raises:
            None (implementations should return error in tuple instead).
        """
        ...



"""Domain.repositories.git_hook_repository
Where: Domain repository interface for git hook configuration.
What: Declares operations to manage git hook scripts and metadata.
Why: Supports infrastructure integration for project git hooks.
"""

#!/usr/bin/env python3
from __future__ import annotations

"""Gitフックリポジトリインターフェース

DDD原則に基づくドメイン層のリポジトリ抽象化
"""


from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from pathlib import Path


class GitHookRepository(ABC):
    """Gitフックリポジトリインターフェース"""

    @abstractmethod
    def install_hooks(self, project_path: Path) -> bool:
        """Gitフックをインストール

        Args:
            project_path: プロジェクトのパス

        Returns:
            インストール成功時True
        """

    @abstractmethod
    def uninstall_hooks(self, project_path: Path) -> bool:
        """Gitフックをアンインストール

        Args:
            project_path: プロジェクトのパス

        Returns:
            アンインストール成功時True
        """

    @abstractmethod
    def is_hooks_installed(self, project_path: Path) -> bool:
        """Gitフックがインストールされているかチェック

        Args:
            project_path: プロジェクトのパス

        Returns:
            インストール済みの場合True
        """

    @abstractmethod
    def get_hook_status(self, project_path: Path) -> dict[str, bool]:
        """各フックの状態を取得

        Args:
            project_path: プロジェクトのパス

        Returns:
            フック名と状態のマッピング
        """

    @abstractmethod
    def update_hooks(self, project_path: Path) -> bool:
        """Gitフックを更新

        Args:
            project_path: プロジェクトのパス

        Returns:
            更新成功時True
        """



"""改稿ロールバック/適用ツール

artifact_id で指定された本文を原稿ファイルへ適用する。
オプションで dry_run（差分のみ）・バックアップ作成・出力先変更に対応。
"""
from __future__ import annotations

import difflib
from pathlib import Path
from typing import Any

from mcp_servers.noveler.domain.entities.mcp_tool_base import (
    MCPToolBase,
    ToolIssue,
    ToolRequest,
    ToolResponse,
)
from noveler.domain.services.artifact_store_service import create_artifact_store
from noveler.infrastructure.adapters.path_service_adapter import create_path_service
from noveler.infrastructure.logging.unified_logger import get_logger


class RestoreManuscriptFromArtifactTool(MCPToolBase):
    def __init__(self) -> None:
        super().__init__(
            tool_name="restore_manuscript_from_artifact",
            tool_description="artifact_idで指定した本文を原稿へ適用（dry_run/backup対応）",
        )

    def get_input_schema(self) -> dict[str, Any]:
        schema = self._get_common_input_schema()
        schema["properties"].update(
            {
                "artifact_id": {
                    "type": "string",
                    "description": "適用する本文のartifact:XXXXXXXX 参照ID",
                },
                "file_path": {
                    "type": "string",
                    "description": "適用先ファイル（省略時はepisodeから推定）",
                },
                "output_path": {
                    "type": "string",
                    "description": "適用先を別ファイルに出力（省略時は上書き）",
                },
                "dry_run": {
                    "type": "boolean",
                    "default": True,
                    "description": "Trueなら差分のみ生成し、ファイルは変更しない",
                },
                "create_backup": {
                    "type": "boolean",
                    "default": True,
                    "description": "上書き時にバックアップを作成",
                },
            }
        )
        schema["required"] = ["episode_number", "artifact_id"]
        return schema

    def execute(self, request: ToolRequest) -> ToolResponse:
        logger = get_logger(__name__)
        import time as _t
        start = _t.time()

        try:
            self._validate_request(request)
            ap = request.additional_params or {}
            artifact_id = str(ap.get("artifact_id", "")).strip()
            if not artifact_id:
                raise ValueError("artifact_id is required")

            dry_run = bool(ap.get("dry_run", True))
            create_backup = bool(ap.get("create_backup", True))

            # 対象パス解決
            fp = self._resolve_target_path(request)
            if isinstance(ap.get("file_path"), str):
                fp = Path(ap["file_path"]).expanduser()
            if fp is None:
                raise FileNotFoundError("target manuscript file not resolved")
            file_path: Path = fp

            # アーティファクト取得
            ps = create_path_service()
            store = create_artifact_store(storage_dir=ps.get_noveler_output_dir() / "artifacts")
            restored = store.fetch(artifact_id)
            if restored is None:
                raise FileNotFoundError(f"artifact not found: {artifact_id}")

            before = file_path.read_text(encoding="utf-8") if file_path.exists() else ""
            after = restored

            # diff
            diff_text = self._make_diff(before, after, file_path)
            diff_ref = store.create_reference(diff_text, alias="rollback_diff", content_type="text", source_file=str(file_path))

            written_to = None
            backup_path = None
            if not dry_run:
                # 出力先
                out_path = Path(ap["output_path"]).expanduser() if isinstance(ap.get("output_path"), str) else file_path
                out_path.parent.mkdir(parents=True, exist_ok=True)
                # バックアップ
                if create_backup and out_path.exists():
                    backup_dir = ps.get_backup_dir()
                    backup_path = backup_dir / f"{out_path.name}.bak"
                    backup_path.write_text(before, encoding="utf-8")
                out_path.write_text(after, encoding="utf-8")
                written_to = str(out_path)

            issue = ToolIssue(
                type="restore_apply_result",
                severity="low",
                message=f"artifact適用{'（dry-run）' if dry_run else ''}",
                file_path=str(file_path),
                details={
                    "artifact_id": artifact_id,
                    "written_to": written_to,
                    "backup_path": str(backup_path) if backup_path else None,
                    "diff_artifact": diff_ref.get("artifact_id"),
                },
            )

            resp = self._create_response(True, 100.0, [issue], start)
            resp.metadata.update(
                {
                    "file_path": str(file_path),
                    "written_to": written_to,
                    "backup_path": str(backup_path) if backup_path else None,
                    "diff_artifact": diff_ref,
                }
            )
            # フォールバックメタ
            self._ps_collect_fallback(ps)
            self._apply_fallback_metadata(resp)
            return resp

        except Exception as e:
            return self._create_response(False, 0.0, [], start, f"restore error: {e!s}")

    # ---- helpers ----
    def _resolve_target_path(self, request: ToolRequest) -> Path | None:
        ap = request.additional_params or {}
        if isinstance(ap.get("file_path"), str):
            return Path(ap["file_path"]).expanduser()
        try:
            from noveler.infrastructure.caching.file_cache_service import get_file_cache_service
            ps = create_path_service()
            cache = get_file_cache_service()
            md = ps.get_manuscript_dir()
            ep = cache.get_episode_file_cached(md, request.episode_number)
            self._ps_collect_fallback(ps)
            return ep
        except Exception:
            return None

    def _make_diff(self, before: str, after: str, path: Path) -> str:
        if before == after:
            return ""
        diff = difflib.unified_diff(
            before.splitlines(keepends=True),
            after.splitlines(keepends=True),
            fromfile=str(path),
            tofile=str(path),
        )
        return "".join(list(diff))



# File: src/noveler/domain/adapters/__init__.py
# Purpose: Domain adapter layer for bridging legacy and new schemas
# Context: Provides adapters for CharacterProfile and other domain objects

"""Domain adapters for schema transformation and compatibility."""

from noveler.domain.adapters.character_profile_adapter import (
    CharacterBookEntry,
    CharacterProfileAdapter,
)

__all__ = [
    "CharacterBookEntry",
    "CharacterProfileAdapter",
]



"""
File: tests/fixtures/anemic_examples.py
Purpose: Intentionally anemic domain models for testing detection
Context: Test cases for anemic domain detection scripts
"""

from dataclasses import dataclass

# ===== ANEMIC EXAMPLES (Should be detected) =====

@dataclass
class AnemicEpisode:
    """
    貧血症のEntity例

    問題: ビジネスロジックが一切ない
    """
    episode_number: int
    title: str
    content: str
    status: str


@dataclass
class AnemicEpisodeNumber:
    """
    貧血症のValue Object例

    問題: バリデーションがない
    """
    value: int


# ===== RICH DOMAIN EXAMPLES (Should pass) =====

@dataclass
class RichEpisode:
    """
    豊かなドメインモデルの例

    改善: ビジネスロジックをカプセル化
    """
    episode_number: int
    title: str
    content: str
    status: str

    def __post_init__(self):
        """バリデーション"""
        if not 1 <= self.episode_number <= 9999:
            raise ValueError(f"Invalid episode number: {self.episode_number}")
        if not self.title:
            raise ValueError("Title cannot be empty")

    def complete(self) -> None:
        """ビジネスロジック: 完了状態に遷移"""
        if self.status == "completed":
            raise ValueError("Episode is already completed")
        self.status = "completed"

    def is_draft(self) -> bool:
        """ビジネスロジック: 下書き状態か判定"""
        return self.status == "draft"


@dataclass
class RichEpisodeNumber:
    """
    豊かなValue Objectの例

    改善: 不変性とバリデーションを保証
    """
    value: int

    def __post_init__(self):
        """バリデーション"""
        if not 1 <= self.value <= 9999:
            raise ValueError(f"Invalid episode number: {self.value}")

    def __eq__(self, other):
        """等価性チェック"""
        if not isinstance(other, RichEpisodeNumber):
            return False
        return self.value == other.value

    def __hash__(self):
        """ハッシュ化可能"""
        return hash(self.value)


# ===== PROTOCOL/INTERFACE EXAMPLES (Should be excluded) =====

class IEpisodeRepository:
    """
    プロトコル定義（検知対象外）

    インターフェースなのでビジネスロジックがなくて当然
    """
    def save(self, episode: RichEpisode) -> None:
        raise NotImplementedError

    def find_by_number(self, episode_number: int) -> RichEpisode | None:
        raise NotImplementedError



#!/usr/bin/env python3
"""統合版 hierarchical_config.py テストケース
test_hierarchical_config.py, test_hierarchical_config_fixed.py, test_hierarchical_config_public.py を統合


仕様書: SPEC-INFRASTRUCTURE
"""

import os
import sys
from pathlib import Path
from unittest.mock import patch

import pytest
import yaml

# テスト対象をインポート
# PathManager統一パス管理システムを使用(事前にパスを設定)
sys.path.insert(0, str(Path(__file__).parent.parent))  # scripts/を追加
from noveler.infrastructure.utils.path_manager import ensure_imports

ensure_imports()
from noveler.infrastructure.adapters.hierarchical_config_adapter import HierarchicalConfigAdapter


class TestHierarchicalConfigCore:
    """HierarchicalConfigAdapterの基本機能テスト"""

    @pytest.fixture
    def temp_dir(self, tmp_path):
        """一時ディレクトリの作成"""
        return tmp_path

    @pytest.fixture
    def config_with_project(self, temp_dir: object):
        """プロジェクトルートありの設定"""
        project_root = temp_dir / "test_project"
        project_root.mkdir()

        # プロジェクト設定ファイルを作成
        config_file = project_root / "プロジェクト設定.yaml"
        config_data = {
            "project": {
                "name": "テストプロジェクト",
                "version": "1.0.0",
            },
        }
        with Path(config_file).open("w", encoding="utf-8") as f:
            yaml.dump(config_data, f, allow_unicode=True)

        return HierarchicalConfigAdapter(project_root)

    @pytest.fixture
    def config_without_project(self):
        """プロジェクトルートなしの設定"""
        return HierarchicalConfigAdapter(None)

    def test_init_with_project_root(self, temp_dir) -> None:
        """プロジェクトルートありでの初期化"""
        config = HierarchicalConfigAdapter(temp_dir)
        assert config.project_root == temp_dir

    def test_init_without_project_root(self) -> None:
        """プロジェクトルートなしでの初期化"""
        config = HierarchicalConfigAdapter()
        assert config.project_root is None

    def test_find_project_root_found(self, temp_dir) -> None:
        """プロジェクトルートの検出(見つかる場合)"""
        # プロジェクト設定ファイルを作成
        config_file = temp_dir / "プロジェクト設定.yaml"
        config_file.touch()

        # サブディレクトリから検索
        sub_dir = temp_dir / "sub" / "dir"
        sub_dir.mkdir(parents=True)

        with patch("pathlib.Path.cwd", return_value=sub_dir):
            config = HierarchicalConfigAdapter()
            assert config.project_root == temp_dir

    def test_find_project_root_not_found(self, temp_dir) -> None:
        """プロジェクトルートの検出(見つからない場合)"""
        with patch("pathlib.Path.cwd", return_value=temp_dir):
            config = HierarchicalConfigAdapter()
            assert config.project_root is None

    def test_get_default_config(self, config_without_project) -> None:
        """デフォルト設定の取得"""
        all_config = config_without_project.all()
        # デフォルト設定がマージされた設定に含まれることを確認
        assert "quality_management" in all_config
        assert "default_threshold" in all_config["quality_management"]
        assert all_config["quality_management"]["default_threshold"] == 80

    def test_load_env_config(self) -> None:
        """環境変数からの設定読み込み"""
        with patch.dict(os.environ, {"NOVEL_QUALITY_THRESHOLD": "80", "OTHER_VAR": "value"}):
            # 環境変数を再読み込みするために新しいインスタンスを作成
            config = HierarchicalConfigAdapter()
            all_config = config.all()
            # 環境変数の設定が反映されていることを確認
            # NOVEL_QUALITY_THRESHOLD -> quality.threshold
            assert "quality" in all_config
            assert all_config["quality"]["threshold"] == 80

    def test_get_all_config(self, config_with_project) -> None:
        """全設定の取得"""
        all_config = config_with_project.all()
        assert isinstance(all_config, dict)
        assert "project" in all_config
        assert all_config["project"]["name"] == "テストプロジェクト"

    def test_get_specific_key(self, config_with_project) -> None:
        """特定キーの取得"""
        # 存在するキー
        value = config_with_project.get("project.name")
        assert value == "テストプロジェクト"

        # 存在しないキー(デフォルト値なし)
        value = config_with_project.get("nonexistent.key")
        assert value is None

        # 存在しないキー(デフォルト値あり)
        value = config_with_project.get("nonexistent.key", "default")
        assert value == "default"

    def test_set_and_save(self, temp_dir) -> None:
        """設定の変更と保存"""
        config = HierarchicalConfigAdapter(temp_dir)

        # プロジェクト設定を変更
        config.set("test.value", 42, level="project")
        config.save("project")

        # 設定ファイルが作成されたことを確認
        config_file = temp_dir / "プロジェクト設定.yaml"
        assert config_file.exists()

        # 内容を確認
        with Path(config_file).open(encoding="utf-8") as f:
            saved_data = yaml.safe_load(f)
        assert saved_data["test"]["value"] == 42

    def test_merge_configs_behavior(self, tmp_path) -> None:
        """設定のマージ動作"""
        # プロジェクト設定を作成
        config_file = tmp_path / "プロジェクト設定.yaml"
        project_config = {"a": 1, "b": {"c": 2}}
        with Path(config_file).open("w", encoding="utf-8") as f:
            yaml.dump(project_config, f, allow_unicode=True)

        # 環境変数を設定してマージ動作を確認
        with patch.dict(os.environ, {"NOVEL_B_D": "3", "NOVEL_E": "4"}):
            config = HierarchicalConfigAdapter(tmp_path)
            merged = config.all()

            # プロジェクト設定が含まれる
            assert merged["a"] == 1
            assert merged["b"]["c"] == 2

            # 環境変数からの設定がマージされる
            # NOVEL_B_D -> b.d
            assert "b" in merged
            assert merged["b"].get("d") == 3
            # NOVEL_E -> e
            assert merged.get("e") == 4

    def test_type_conversion(self) -> None:
        """型変換のテスト"""
        with patch.dict(
            os.environ,
            {
                "NOVEL_QUALITY_THRESHOLD": "85",
                "NOVEL_DEBUG": "true",
                "NOVEL_MAX_RETRIES": "3",
            },
        ):
            config = HierarchicalConfigAdapter()
            config.all()

            # 環境変数経由の値が適切に変換されることを確認
            # NOVEL_QUALITY_THRESHOLD -> quality.threshold (数値に変換)
            assert config.get("quality.threshold") == 85
            # NOVEL_DEBUG -> debug (ブール値に変換)
            assert config.get("debug") is True
            # NOVEL_MAX_RETRIES -> max.retries (数値に変換)
            assert config.get("max.retries") == 3


class TestHierarchicalConfigPublicAPI:
    """公開APIのテスト"""

    @pytest.fixture
    def temp_project(self, tmp_path):
        """テスト用プロジェクトの作成"""
        project_dir = tmp_path / "test_project"
        project_dir.mkdir()

        # プロジェクト設定
        config_file = project_dir / "プロジェクト設定.yaml"
        config_data = {
            "project": {
                "title": "テスト小説",
                "author": "太郎",
            },
            "quality": {
                "threshold": 75,
            },
        }
        with Path(config_file).open("w", encoding="utf-8") as f:
            yaml.dump(config_data, f, allow_unicode=True)

        return project_dir

    def test_basic_usage(self, temp_project) -> None:
        """基本的な使用方法"""
        config = HierarchicalConfigAdapter(temp_project)

        # getメソッド
        assert config.get("project.title") == "テスト小説"
        assert config.get("project.author") == "太郎"
        assert config.get("quality.threshold") == 75

    def test_environment_override(self, temp_project) -> None:
        """環境変数による上書き"""
        with patch.dict(os.environ, {"NOVEL_QUALITY_THRESHOLD": "90"}):
            config = HierarchicalConfigAdapter(temp_project)

            # 環境変数が優先される (NOVEL_QUALITY_THRESHOLD -> quality.threshold)
            assert config.get("quality.threshold") == 90
            # プロジェクト設定も残っている(但し環境変数で上書きされる)
            # プロジェクト設定のみの場合のテスト
            with patch.dict(os.environ, {}, clear=True):
                config2 = HierarchicalConfigAdapter(temp_project)
                assert config2.get("quality.threshold") == 75

    def test_nested_get_with_dot_notation(self, temp_project) -> None:
        """ドット記法によるネストされた値の取得"""
        config = HierarchicalConfigAdapter(temp_project)

        # 深いネストのテスト
        config.set("deep.nested.value", "test", level="project")
        assert config.get("deep.nested.value") == "test"

    def test_error_handling(self) -> None:
        """エラーハンドリング"""
        config = HierarchicalConfigAdapter()

        # 無効なレベルでの保存
        with pytest.raises(ValueError, match=".*"):
            config.set("test", "value", level="invalid")
            config.save("invalid")


class TestHierarchicalConfigAdvanced:
    """高度な機能のテスト"""

    def test_init_global_config(self, tmp_path, monkeypatch) -> None:
        """グローバル設定の初期化"""
        # ホームディレクトリをテンポラリに変更
        home_dir = tmp_path / "home"
        home_dir.mkdir()
        monkeypatch.setattr(Path, "home", lambda: home_dir)

        config = HierarchicalConfigAdapter()

        # 設定が正常に作成されることを確認(ディレクトリ作成は必須ではない)
        assert config is not None
        assert config.all() is not None

    def test_show_config_sources(self, capsys) -> None:
        """設定ソースの表示"""
        config = HierarchicalConfigAdapter()
        # show_sourcesメソッドが存在しない場合は、get_allで代替
        if hasattr(config, "show_sources"):
            config.show_sources()
            captured = capsys.readouterr()
            assert "設定ソース" in captured.out or "プロジェクト設定" in captured.out
        else:
            # show_sourcesメソッドがない場合は基本機能をテスト
            all_config = config.all()
            assert isinstance(all_config, dict)
            # デフォルト設定が含まれることを確認
            assert "quality_management" in all_config or "default_project" in all_config

    def test_empty_configs_handling(self) -> None:
        """空の設定の処理"""
        config = HierarchicalConfigAdapter()

        # 空の設定でもエラーにならない
        assert config.get("nonexistent") is None
        assert config.all() is not None

    def test_deep_nested_config(self) -> None:
        """深くネストされた設定"""
        config = HierarchicalConfigAdapter()

        # setメソッドが存在しない場合は、getの基本機能をテスト
        if hasattr(config, "set") and hasattr(config, "save"):
            # 深いネストの設定
            config.set("level1.level2.level3.level4", "deep_value", level="project")

            # 取得できることを確認
            assert config.get("level1.level2.level3.level4") == "deep_value"

            # 中間レベルも辞書として取得可能
            level2 = config.get("level1.level2")
            assert isinstance(level2, dict)
            assert level2["level3"]["level4"] == "deep_value"
        else:
            # setメソッドがない場合は基本機能をテスト
            all_config = config.all()
            assert isinstance(all_config, dict)
            # デフォルト設定が含まれることを確認
            assert "quality_management" in all_config or len(all_config) >= 0


class TestMorphologicalAnalyzer:
    """形態素解析器テストの統合"""

    @pytest.fixture
    def sample_text(self) -> str:
        """テスト用サンプルテキスト"""
        return "太郎は花子に本を渡しました。とても嬉しそうでした。"

    def test_basic_analysis(self, sample_text) -> None:
        """基本的な形態素解析"""
        # MorphologicalAnalyzerのテストをここに統合
        # 実際の実装に応じて調整が必要
        assert sample_text is not None

    def test_pos_distribution(self, sample_text) -> None:
        """品詞分布の分析"""
        # 品詞分布テストの実装
        assert sample_text is not None

    def test_sentence_endings(self, sample_text) -> None:
        """文末表現の抽出"""
        # 文末表現テストの実装
        assert sample_text is not None

    def test_readability_metrics(self, sample_text) -> None:
        """読みやすさ指標の計算"""
        # 読みやすさ指標テストの実装
        assert sample_text is not None


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



#!/usr/bin/env python3
"""Domain用DIコンテナファクトリー実装

Infrastructure層でDomain抽象化を実装
Phase 4: Domain純粋性完全確保
"""

from typing import TypeVar

from noveler.domain.interfaces.di_container_factory import DIResolutionError, IDIContainerFactory
from noveler.infrastructure.di.simple_di_container import configure_dependencies, get_container

T = TypeVar("T")


class DomainDIContainerFactory(IDIContainerFactory):
    """Domain層用DIコンテナファクトリー実装

    simple_di_containerのDomain抽象化ラッパー
    Domain層からのInfrastructure直接依存を除去
    """

    def __init__(self) -> None:
        """初期化"""
        configure_dependencies()
        self._container = get_container()

    def resolve(self, interface_type: type[T]) -> T:
        """インターフェース型から実装を解決

        Args:
            interface_type: 解決したいインターフェース型

        Returns:
            interface_typeの実装インスタンス

        Raises:
            DIResolutionError: 解決できない場合
        """
        try:
            return self._container.get(interface_type)
        except (ValueError, KeyError) as e:
            raise DIResolutionError(interface_type, f"DI container resolution failed: {e}") from e

    def is_registered(self, interface_type: type) -> bool:
        """指定したインターフェースが登録されているか確認

        Args:
            interface_type: 確認したいインターフェース型

        Returns:
            bool: 登録されている場合True
        """
        try:
            self._container.get(interface_type)
            return True
        except (ValueError, KeyError):
            return False


# グローバルインスタンス（Domain層からアクセス可能）
_domain_di_factory: IDIContainerFactory | None = None


def get_domain_di_factory() -> IDIContainerFactory:
    """Domain用DIファクトリーの取得

    Domain層のエンティティ・サービスからアクセス可能な
    Infrastructure非依存のDI抽象化層

    Returns:
        IDIContainerFactory: Domain用DIファクトリー
    """
    global _domain_di_factory

    if _domain_di_factory is None:
        try:
            _domain_di_factory = DomainDIContainerFactory()
        except ImportError:
            # Infrastructure層が利用できない場合のフォールバック
            from noveler.domain.interfaces.di_container_factory import NullDIContainerFactory

            _domain_di_factory = NullDIContainerFactory()

    return _domain_di_factory


def set_domain_di_factory(factory: IDIContainerFactory) -> None:
    """Domain用DIファクトリーの設定

    テスト時やカスタム実装を使用する場合に使用

    Args:
        factory: 設定するDIファクトリー
    """
    global _domain_di_factory
    _domain_di_factory = factory



# File: scripts/ci/ensure_dist_wrapper.py
# Purpose: Ensure dist/mcp_servers/noveler/main.py exists for CI/tests.
# Context: Some tests and MCP configs expect a dist wrapper path to exist.
#          This script generates a minimal, safe wrapper that delegates
#          to the src entrypoint without requiring a full production build.

"""CI helper to ensure dist MCP wrapper exists.

This script creates `dist/mcp_servers/noveler/main.py` if missing and writes a
thin wrapper that imports the source entrypoint (`mcp_servers.noveler.main`) by
temporarily adding the project `src/` directory to `sys.path`.

It keeps behavior minimal and reversible:
- Does not modify other dist assets.
- Idempotent: safe to run multiple times.

Side effects:
- Creates directories under `dist/mcp_servers/noveler/` if they do not exist.
"""

from __future__ import annotations

import os
from pathlib import Path


def main() -> int:
    project_root = Path(__file__).resolve().parents[2]
    dist_dir = project_root / "dist" / "mcp_servers" / "noveler"
    dist_dir.mkdir(parents=True, exist_ok=True)

    wrapper_path = dist_dir / "main.py"

    # Minimal, robust wrapper that executes the src entrypoint.
    content = """#!/usr/bin/env python3
from __future__ import annotations
import sys, os
from pathlib import Path

# Resolve project root and ensure src is importable
PROJECT_ROOT = Path(__file__).resolve().parents[3]
SRC_DIR = PROJECT_ROOT / 'src'
if str(SRC_DIR) not in sys.path:
    sys.path.insert(0, str(SRC_DIR))

os.environ.setdefault('PYTHONUNBUFFERED', '1')

def _run() -> int:
    try:
        import asyncio
        from importlib import import_module
        src_main = import_module('mcp_servers.noveler.main')
        if hasattr(src_main, 'main'):
            asyncio.run(src_main.main())
            return 0
    except SystemExit as e:
        return int(getattr(e, 'code', 0) or 0)
    except Exception:
        # Fallback: presentation runtime direct call
        try:
            import asyncio
            from noveler.presentation.mcp import server_runtime
            asyncio.run(server_runtime.main())
            return 0
        except Exception as e:  # pragma: no cover - best-effort wrapper
            print(f"Wrapper failed to start MCP server: {e}")
            return 1
    return 0

if __name__ == '__main__':
    raise SystemExit(_run())
"""

    # Only write if missing or content changed to avoid unnecessary churn
    if not wrapper_path.exists() or wrapper_path.read_text(encoding="utf-8", errors="ignore") != content:
        wrapper_path.write_text(content, encoding="utf-8")
        try:
            wrapper_path.chmod(0o755)
        except Exception:
            pass

    # Optional: set env hint for prod runs (harmless for tests)
    os.environ.setdefault("NOVEL_PRODUCTION_MODE", "1")
    return 0


if __name__ == "__main__":
    raise SystemExit(main())



"""Infrastructure.git.hooks.install_b20_hooks
Where: Infrastructure script installing B20 git hooks into a repository.
What: Copies hook scripts and ensures the repo is configured correctly.
Why: Simplifies setup of B20-compliant git hooks for developers.
"""

from noveler.presentation.shared.shared_utilities import console

"B20開発プロセス準拠Git Hooksインストーラー\n\n仕様書: B20開発作業指示書準拠\n"
import argparse
import stat
import sys
from pathlib import Path
from typing import Any

from noveler.infrastructure.logging.unified_logger import get_logger

logger = get_logger(__name__)


class B20HooksInstaller:
    """B20開発プロセス準拠Git Hooksインストーラー"""

    def __init__(self, git_root: Path, guide_root: Path) -> None:
        """初期化"""
        self.git_root = git_root
        self.guide_root = guide_root
        self.hooks_dir = git_root / ".git" / "hooks"

    def install_all_hooks(self, force: bool = False) -> dict[str, Any]:
        """全てのB20 Hooksのインストール"""
        result = {"success": True, "installed_hooks": [], "skipped_hooks": [], "errors": []}
        try:
            self.hooks_dir.mkdir(exist_ok=True)
            pre_commit_result = self.install_pre_commit_hook(force)
            if pre_commit_result["success"]:
                result["installed_hooks"].append("pre-commit")
            elif pre_commit_result.get("skipped"):
                result["skipped_hooks"].append("pre-commit")
            else:
                result["success"] = False
                result["errors"].extend(pre_commit_result["errors"])
            post_commit_result = self.install_post_commit_hook(force)
            if post_commit_result["success"]:
                result["installed_hooks"].append("post-commit")
            elif post_commit_result.get("skipped"):
                result["skipped_hooks"].append("post-commit")
            else:
                result["success"] = False
                result["errors"].extend(post_commit_result["errors"])
            commit_msg_result = self.install_commit_msg_hook(force)
            if commit_msg_result["success"]:
                result["installed_hooks"].append("commit-msg")
            elif commit_msg_result.get("skipped"):
                result["skipped_hooks"].append("commit-msg")
            else:
                result["success"] = False
                result["errors"].extend(commit_msg_result["errors"])
        except Exception as e:
            result["success"] = False
            result["errors"].append(f"Hooksインストールエラー: {e!s}")
        return result

    def install_pre_commit_hook(self, force: bool = False) -> dict[str, Any]:
        """Pre-commit hookのインストール"""
        hook_file = self.hooks_dir / "pre-commit"
        if hook_file.exists() and (not force):
            return {
                "success": True,
                "skipped": True,
                "errors": [],
                "message": "Pre-commit hook already exists (use --force to overwrite)",
            }
        try:
            hook_content = self._generate_pre_commit_hook_script()
            hook_file.write_text(hook_content, encoding="utf-8")
            hook_file.chmod(hook_file.stat().st_mode | stat.S_IEXEC)
            self.logger_service.info("Pre-commit hook installed: %s", hook_file)
            return {
                "success": True,
                "skipped": False,
                "errors": [],
                "message": "Pre-commit hook installed successfully",
            }
        except Exception as e:
            return {
                "success": False,
                "skipped": False,
                "errors": [f"Pre-commit hook installation failed: {e!s}"],
                "message": None,
            }

    def install_post_commit_hook(self, force: bool = False) -> dict[str, Any]:
        """Post-commit hookのインストール（CODEMAP更新統合）"""
        hook_file = self.hooks_dir / "post-commit"
        if hook_file.exists() and (not force):
            return {
                "success": True,
                "skipped": True,
                "errors": [],
                "message": "Post-commit hook already exists (use --force to overwrite)",
            }
        try:
            hook_content = self._generate_post_commit_hook_script()
            hook_file.write_text(hook_content, encoding="utf-8")
            hook_file.chmod(hook_file.stat().st_mode | stat.S_IEXEC)
            self.logger_service.info("Post-commit hook installed: %s", hook_file)
            return {
                "success": True,
                "skipped": False,
                "errors": [],
                "message": "Post-commit hook installed successfully",
            }
        except Exception as e:
            return {
                "success": False,
                "skipped": False,
                "errors": [f"Post-commit hook installation failed: {e!s}"],
                "message": None,
            }

    def install_commit_msg_hook(self, force: bool = False) -> dict[str, Any]:
        """Commit-msg hookのインストール"""
        hook_file = self.hooks_dir / "commit-msg"
        if hook_file.exists() and (not force):
            return {
                "success": True,
                "skipped": True,
                "errors": [],
                "message": "Commit-msg hook already exists (use --force to overwrite)",
            }
        try:
            hook_content = self._generate_commit_msg_hook_script()
            hook_file.write_text(hook_content, encoding="utf-8")
            hook_file.chmod(hook_file.stat().st_mode | stat.S_IEXEC)
            self.logger_service.info("Commit-msg hook installed: %s", hook_file)
            return {
                "success": True,
                "skipped": False,
                "errors": [],
                "message": "Commit-msg hook installed successfully",
            }
        except Exception as e:
            return {
                "success": False,
                "skipped": False,
                "errors": [f"Commit-msg hook installation failed: {e!s}"],
                "message": None,
            }

    def uninstall_hooks(self) -> dict[str, Any]:
        """B20 Hooksのアンインストール"""
        result = {"success": True, "uninstalled_hooks": [], "errors": []}
        hooks_to_remove = ["pre-commit", "post-commit", "commit-msg"]
        for hook_name in hooks_to_remove:
            hook_file = self.hooks_dir / hook_name
            if hook_file.exists():
                try:
                    content = hook_file.read_text(encoding="utf-8")
                    if "B20開発プロセス" in content or "B20PreCommitValidator" in content:
                        hook_file.unlink()
                        result["uninstalled_hooks"].append(hook_name)
                        self.logger_service.info("B20 hook uninstalled: %s", hook_name)
                    else:
                        self.logger_service.warning("Skipped non-B20 hook: %s", hook_name)
                except Exception as e:
                    result["success"] = False
                    result["errors"].append(f"Failed to uninstall {hook_name}: {e!s}")
        return result

    def get_installation_status(self) -> dict[str, Any]:
        """インストール状態の取得"""
        status = {
            "git_root": str(self.git_root),
            "guide_root": str(self.guide_root),
            "hooks_dir": str(self.hooks_dir),
            "hooks": {},
        }
        hooks_to_check = ["pre-commit", "post-commit", "commit-msg"]
        for hook_name in hooks_to_check:
            hook_file = self.hooks_dir / hook_name
            if hook_file.exists():
                try:
                    content = hook_file.read_text(encoding="utf-8")
                    is_b20_hook = "B20開発プロセス" in content or "B20PreCommitValidator" in content
                    status["hooks"][hook_name] = {
                        "exists": True,
                        "is_b20_hook": is_b20_hook,
                        "executable": hook_file.stat().st_mode & stat.S_IEXEC != 0,
                        "size": hook_file.stat().st_size,
                    }
                except Exception as e:
                    status["hooks"][hook_name] = {"exists": True, "error": f"読み取りエラー: {e!s}"}
            else:
                status["hooks"][hook_name] = {"exists": False}
        return status

    def _generate_pre_commit_hook_script(self) -> str:
        """Pre-commit hook スクリプトの生成"""
        return f'#!/usr/bin/env python3\n"""B20開発プロセス準拠 Pre-commit Hook\n\nAuto-generated by B20HooksInstaller\n"""\n\nimport sys\nimport os\nfrom pathlib import Path\n\n# ガイドルートをPythonパスに追加\nGUIDE_ROOT = Path("{self.guide_root}")\nsys.path.insert(0, str(GUIDE_ROOT))\n\ntry:\n\n    # Pre-commit検証の実行\n    exit_code = main()\n    sys.exit(exit_code)\n\nexcept ImportError as e:\n    self.console_service.print(f"❌ B20 Pre-commit hook import error: {{e}}", file=sys.stderr)\n    self.console_service.print("⚠️ Skipping B20 validation checks", file=sys.stderr)\n    sys.exit(0)  # 警告として扱い、コミットは続行\n\nexcept Exception as e:\n    self.console_service.print(f"❌ B20 Pre-commit hook execution error: {{e}}", file=sys.stderr)\n    self.console_service.print("⚠️ Skipping B20 validation checks", file=sys.stderr)\n    sys.exit(0)  # 警告として扱い、コミットは続行\n'

    def _generate_post_commit_hook_script(self) -> str:
        """Post-commit hook スクリプトの生成（CODEMAP更新統合）"""
        return f'''#!/usr/bin/env python3\n"""B20開発プロセス準拠 Post-commit Hook (CODEMAP更新統合)\n\nAuto-generated by B20HooksInstaller\n"""\n\nimport sys\nimport subprocess\nfrom pathlib import Path\n\n# ガイドルートをPythonパスに追加\nGUIDE_ROOT = Path("{self.guide_root}")\nsys.path.insert(0, str(GUIDE_ROOT))\n\ndef main():\n    """Post-commit処理の実行"""\n    try:\n        # 1. CODEMAP自動更新の実行\n        codemap_update_result = update_codemap()\n\n        # 2. 3コミットサイクル進行の実行\n        cycle_update_result = update_three_commit_cycle()\n\n        # 結果出力\n        if codemap_update_result or cycle_update_result:\n            print("✅ B20 Post-commit processing completed")\n\n    except Exception as e:\n        print(f"⚠️ B20 Post-commit processing warning: {{e}}", file=sys.stderr)\n        # エラーがあってもコミットは成功とする\n\ndef update_codemap():\n    """CODEMAP自動更新の実行"""\n    try:\n        # 既存のCODEMAP更新機能を利用\n\n        hook = CodeMapPostCommitHook(Path("{self.git_root}"))\n        result = hook.execute_codemap_update()\n\n        if result.get("updated"):\n            print(f"📋 CODEMAP updated: {{result.get('changes', '')}}")\n            return True\n\n        return False\n\n    except ImportError:\n        print("⚠️ CODEMAP update skipped (service not available)", file=sys.stderr)\n        return False\n    except Exception as e:\n        print(f"⚠️ CODEMAP update error: {{e}}", file=sys.stderr)\n        return False\n\ndef update_three_commit_cycle():\n    """3コミットサイクル進行の実行"""\n    try:\n        # 最新のコミットハッシュを取得\n        result = subprocess.run([\n            "git", "rev-parse", "HEAD"\n        ], capture_output=True, text=True, cwd=Path("{self.git_root}"))\n\n        if result.returncode != 0:\n            return False\n\n        commit_hash = result.stdout.strip()\n\n        # コミットメッセージから機能名を推測\n        commit_msg_result = subprocess.run([\n            "git", "log", "-1", "--pretty=%B"\n        ], capture_output=True, text=True, cwd=Path("{self.git_root}"))\n\n        if commit_msg_result.returncode != 0:\n            return False\n\n        commit_message = commit_msg_result.stdout.strip()\n        feature_name = extract_feature_name(commit_message)\n\n        if feature_name:\n\n            service = ThreeCommitCycleService(GUIDE_ROOT)\n\n            # サイクル進行の試行\n            try:\n                updated_cycle = service.advance_cycle_stage(feature_name, commit_hash)\n                print(f"🔄 Three-commit cycle advanced: {{feature_name}} -> {{updated_cycle.current_stage.value}}")\n                return True\n            except ValueError as e:\n                # サイクル進行ができない場合（正常なケース）\n                print(f"ℹ️ Three-commit cycle: {{e}}")\n                return False\n\n        return False\n\n    except ImportError:\n        print("⚠️ Three-commit cycle update skipped (service not available)", file=sys.stderr)\n        return False\n    except Exception as e:\n        print(f"⚠️ Three-commit cycle update error: {{e}}", file=sys.stderr)\n        return False\n\ndef extract_feature_name(commit_message):\n    """コミットメッセージから機能名を抽出"""\n    # feat: feature_name のパターンを抽出\n\n    patterns = [\n        r'feat:\\s*([\\w_-]+)',\n        r'fix:\\s*([\\w_-]+)',\n        r'refactor:\\s*([\\w_-]+)',\n        r'docs:\\s*([\\w_-]+)'\n    ]\n\n    for pattern in patterns:\n        match = re.search(pattern, commit_message)\n        if match:\n            return match.group(1)\n\n    return None\n\nif __name__ == "__main__":\n    main()\n'''

    def _generate_commit_msg_hook_script(self) -> str:
        """Commit-msg hook スクリプトの生成"""
        return '#!/usr/bin/env python3\n"""B20開発プロセス準拠 Commit-msg Hook\n\nAuto-generated by B20HooksInstaller\n"""\n\nimport sys\nimport re\nfrom pathlib import Path\n\ndef main():\n    """Commit message検証の実行"""\n    if len(sys.argv) != 2:\n        print("❌ Invalid commit-msg hook usage", file=sys.stderr)\n        sys.exit(1)\n\n    commit_msg_file = Path(sys.argv[1])\n\n    try:\n        commit_message = commit_msg_file.read_text(encoding=\'utf-8\').strip()\n\n        # B20コミットメッセージ規約の検証\n        validation_result = validate_commit_message(commit_message)\n\n        if validation_result["valid"]:\n            if validation_result["warnings"]:\n                for warning in validation_result["warnings"]:\n                    print(f"⚠️ {warning}", file=sys.stderr)\n\n            sys.exit(0)\n        else:\n            print("❌ Commit message validation failed:", file=sys.stderr)\n            for error in validation_result["errors"]:\n                print(f"  - {error}", file=sys.stderr)\n\n            if validation_result["suggestions"]:\n                print("\\n💡 Suggestions:", file=sys.stderr)\n                for suggestion in validation_result["suggestions"]:\n                    print(f"  - {suggestion}", file=sys.stderr)\n\n            sys.exit(1)\n\n    except Exception as e:\n        print(f"❌ Commit message validation error: {e}", file=sys.stderr)\n        sys.exit(0)  # エラー時は警告として扱い、コミットは続行\n\ndef validate_commit_message(message):\n    """コミットメッセージの検証"""\n    result = {\n        "valid": True,\n        "errors": [],\n        "warnings": [],\n        "suggestions": []\n    }\n\n    # 基本的なフォーマット検証\n    conventional_pattern = r\'^(feat|fix|docs|style|refactor|test|chore)(\\(.+\\))?: .+$\'\n\n    if not re.match(conventional_pattern, message, re.MULTILINE):\n        result["valid"] = False\n        result["errors"].append("Conventional Commits format required")\n        result["suggestions"].append("Use format: type(scope): description")\n        result["suggestions"].append("Example: feat(auth): add user authentication")\n\n    # 長さ検証\n    lines = message.split(\'\\n\')\n    if lines[0] and len(lines[0]) > 72:\n        result["warnings"].append("First line exceeds 72 characters")\n\n    # B20 3コミットサイクル検証\n    if "feat:" in message:\n        cycle_validation = validate_three_commit_cycle_message(message)\n        if not cycle_validation["valid"]:\n            result["warnings"].extend(cycle_validation["warnings"])\n\n    return result\n\ndef validate_three_commit_cycle_message(message):\n    """3コミットサイクルメッセージ検証"""\n    cycle_patterns = [\n        r\'仕様書.*設計.*完了\',\n        r\'実装.*テスト.*完了\',\n        r\'ドキュメント.*統合.*完了\'\n    ]\n\n    is_cycle_commit = any(re.search(pattern, message) for pattern in cycle_patterns)\n\n    if is_cycle_commit:\n        return {\n            "valid": True,\n            "warnings": []\n        }\n    else:\n        return {\n            "valid": True,\n            "warnings": ["Consider using 3-commit cycle format for better tracking"]\n        }\n\nif __name__ == "__main__":\n    main()\n'


def install_b20_hooks_cli() -> int:
    """CLIエントリーポイント"""
    parser = argparse.ArgumentParser(description="B20開発プロセス準拠Git Hooksインストーラー")
    parser.add_argument("--git-root", type=Path, required=True, help="Gitリポジトリのルートパス")
    parser.add_argument("--guide-root", type=Path, required=True, help="ガイドプロジェクトのルートパス")
    parser.add_argument("--force", action="store_true", help="既存フックを上書き")
    parser.add_argument("--uninstall", action="store_true", help="B20フックをアンインストール")
    parser.add_argument("--status", action="store_true", help="インストール状態を表示")
    args = parser.parse_args()
    if not args.git_root.exists():
        console.print(f"❌ Git root not found: {args.git_root}", file=sys.stderr)
        return 1
    if not args.guide_root.exists():
        console.print(f"❌ Guide root not found: {args.guide_root}", file=sys.stderr)
        return 1
    installer = B20HooksInstaller(args.git_root, args.guide_root)
    try:
        if args.uninstall:
            result = installer.uninstall_hooks()
            if result["success"]:
                console.print(f"✅ B20 hooks uninstalled: {', '.join(result['uninstalled_hooks'])}")
                return 0
            console.print("❌ Uninstallation failed:", file=sys.stderr)
            for error in result["errors"]:
                console.print(f"  - {error}", file=sys.stderr)
            return 1
        if args.status:
            status = installer.get_installation_status()
            console.print("📊 B20 Hooks Installation Status")
            console.print(f"Git Root: {status['git_root']}")
            console.print(f"Guide Root: {status['guide_root']}")
            console.print(f"Hooks Directory: {status['hooks_dir']}")
            console.print()
            for hook_name, hook_status in status["hooks"].items():
                if hook_status.get("exists"):
                    status_icon = "✅" if hook_status.get("is_b20_hook") else "⚠️"
                    exec_status = "executable" if hook_status.get("executable") else "not executable"
                    console.print(f"{status_icon} {hook_name}: {exec_status}")
                    if hook_status.get("error"):
                        console.print(f"   Error: {hook_status['error']}")
                else:
                    console.print(f"❌ {hook_name}: not installed")
            return 0
        result = installer.install_all_hooks(args.force)
        if result["success"]:
            console.print("✅ B20 hooks installation completed")
            if result["installed_hooks"]:
                console.print(f"Installed: {', '.join(result['installed_hooks'])}")
            if result["skipped_hooks"]:
                console.print(f"Skipped: {', '.join(result['skipped_hooks'])} (use --force to overwrite)")
            return 0
        console.print("❌ Installation failed:", file=sys.stderr)
        for error in result["errors"]:
            console.print(f"  - {error}", file=sys.stderr)
        return 1
    except Exception as e:
        console.print(f"❌ Installation error: {e}", file=sys.stderr)
        return 1


if __name__ == "__main__":
    sys.exit(install_b20_hooks_cli())



# File: tests/unit/domain/services/test_manuscript_fetch_service.py
# Purpose: Verify manuscript fetch fallback workflow and QC codes.
# Context: Implements SPEC-QUALITY-120 manuscript fetch requirements.

from __future__ import annotations

import hashlib
from typing import Any

import pytest

from noveler.domain.services.manuscript_fetch_service import (
    ManuscriptFetchError,
    ManuscriptFetchResult,
    ManuscriptFetchService,
    ManuscriptToolError,
    ManuscriptToolResponse,
)


def sha(text: str) -> str:
    return hashlib.sha256(text.encode("utf-8")).hexdigest()


class DummyTool:
    def __init__(self, *, response: ManuscriptToolResponse | None = None, error: ManuscriptToolError | None = None):
        self.response = response
        self.error = error
        self.calls: list[dict[str, Any]] = []

    def __call__(self, *, manuscript_hash: str) -> ManuscriptToolResponse:
        self.calls.append({"manuscript_hash": manuscript_hash})
        if self.error is not None:
            raise self.error
        if self.response is None:
            raise ManuscriptToolError("no response", code="QC-016")
        return self.response


def test_fetch_artifact_success_first_try():
    content = "本文データ"
    tool = DummyTool(response=ManuscriptToolResponse(content=content, manuscript_hash=sha(content)))
    service = ManuscriptFetchService(
        fetch_artifact=tool,
        read_snapshot=DummyTool(error=ManuscriptToolError("unused", code="QC-015")),
        request_manual_upload=DummyTool(error=ManuscriptToolError("unused", code="QC-017")),
        sleep=lambda _: None,
    )

    result = service.fetch_excerpt(manuscript_hash=sha(content))

    assert isinstance(result, ManuscriptFetchResult)
    assert result.excerpt == content
    assert result.tool_id == "fetch_artifact"
    assert result.metadata["excerpt_hash"] == sha(content)
    assert result.attempts[0].result == "success"
    assert result.attempts[0].tool_id == "fetch_artifact"


def test_fallback_to_read_snapshot_on_cache_miss():
    cache_miss_tool = DummyTool(error=ManuscriptToolError("cache miss", code="QC-015"))
    content = "最新本文"
    snapshot_tool = DummyTool(response=ManuscriptToolResponse(content=content, manuscript_hash=sha(content)))
    manual_tool = DummyTool(error=ManuscriptToolError("pending", code="QC-017"))
    delays: list[float] = []

    service = ManuscriptFetchService(
        fetch_artifact=cache_miss_tool,
        read_snapshot=snapshot_tool,
        request_manual_upload=manual_tool,
        sleep=lambda sec: delays.append(sec),
    )

    result = service.fetch_excerpt(manuscript_hash=sha(content))

    assert result.tool_id == "read_snapshot"
    assert len(delays) == 1 and delays[0] == 1.0
    assert result.attempts[0].qc_code == "QC-015"
    assert result.attempts[1].result == "success"


def test_hash_mismatch_then_manual_upload_failure():
    wrong_content = "古い本文"
    hash_mismatch_tool = DummyTool(response=ManuscriptToolResponse(content=wrong_content, manuscript_hash=sha("別")))
    storage_fail_tool = DummyTool(error=ManuscriptToolError("storage", code="QC-016"))
    pending_tool = DummyTool(error=ManuscriptToolError("pending", code="QC-017"))

    service = ManuscriptFetchService(
        fetch_artifact=hash_mismatch_tool,
        read_snapshot=storage_fail_tool,
        request_manual_upload=pending_tool,
        sleep=lambda _: None,
    )

    with pytest.raises(ManuscriptFetchError) as exc_info:
        service.fetch_excerpt(manuscript_hash=sha("期待"))

    err = exc_info.value
    assert err.code == "QC-017"
    qc_codes = [attempt.get("qc_code") for attempt in err.attempts]
    assert qc_codes == ["QC-018", "QC-016", "QC-017"]


def test_preferred_tools_reorders_priorities():
    first = DummyTool(response=ManuscriptToolResponse(content="A", manuscript_hash=sha("A")))
    second = DummyTool(response=ManuscriptToolResponse(content="B", manuscript_hash=sha("B")))
    third = DummyTool(response=ManuscriptToolResponse(content="C", manuscript_hash=sha("C")))

    service = ManuscriptFetchService(fetch_artifact=first, read_snapshot=second, request_manual_upload=third, sleep=lambda _: None)

    result = service.fetch_excerpt(manuscript_hash=sha("B"), preferred_tools=["read_snapshot"])

    assert result.tool_id == "read_snapshot"
    assert first.calls == []
